unit UCDefine;

interface

uses
{$IFDEF VER130}
  dsgnintf, windows,
{$ELSE}
  Variants, ExtActns, UCMail,
  {$IFDEF VER150}
  ActnMan, ActnMenus, DesignIntf, DesignEditors,
  {$ELSE}
    {$IFDEF VER140}
      DesignIntf, DesignEditors,    
    {$ENDIF}
  {$ENDIF}

{$ENDIF}
  SysUtils, Classes, Menus, ActnList, UCMessages, Controls,
  UCConsts,  Graphics, DB, Forms, LoginWindow_U, ViewLog_U;


function Decrypt(const S: AnsiString; Key: Word): AnsiString;
function Encrypt(const S: AnsiString; Key: Word): AnsiString;

{$IFDEF Ver130}
function StrToBool ( Valor : String) : Boolean;
function BoolToStr ( Valor : Boolean) : String;
{$ENDIF}

type
  TUserDef = class(TComponent) //classe para armazenar usuario logado = currentuser
    public
     UserID, Profile : Integer;
     Username, LoginName, Password, Email : String;
     Privilegiado : Boolean;
  end;

  TCadastroUsuarios = class(TPersistent) // armazenar menuitem ou action responsavel pelo cadastro de usuarios
    private
      FActCadUsu: TAction;
      FMenuCadUsu: TMenuItem;
      FUsaPriv: Boolean;
      FLockADM: Boolean;
      procedure SetActCadUsu(const Value: TAction);
      procedure SetMenuCadUsu(const Value: TMenuItem);
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property Action : TAction read FActCadUsu write SetActCadUsu;
      property MenuItem : TMenuItem read FMenuCadUsu write SetMenuCadUsu;
      property UsePrivilegedField : Boolean read FUsaPriv write FUsaPriv;
      property ProtectAdmin : Boolean read FLockADM write FLockADM;
  end;

  TPerfilUsuarios = class(TPersistent) // armazenar menuitem ou action responsavel pelo Perfil de usuarios
    private
      FPerfUsuAtive: Boolean;
      FActPerfUsu: TAction;
      FMenuPerfUsu: TMenuItem;
      procedure SetActPerfUsu(const Value: TAction);
      procedure SetMenuPerfUsu(const Value: TMenuItem);
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property Active : Boolean read FPerfUsuAtive write FPerfUsuAtive;
      property Action : TAction read FActPerfUsu write SetActPerfUsu;
      property MenuItem : TMenuItem read FMenuPerfUsu write SetMenuPerfUsu;
  end;

  TTrocarSenha = class(TPersistent) // armazenar menuitem ou action responsavel pelo Form trocar senha
    private
      FForcePass: Boolean;
      FPassLen: Integer;
      FActChgPass: TAction;
      FMenuChgPassUsu: TMenuItem;
      procedure SetActChgPassUsu(const Value: TAction);
      procedure SetMenuChgPassUsu(const Value: TMenuItem);
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property Action : TAction read FActChgPass write SetActChgPassUsu;
      property MenuItem : TMenuItem read FMenuChgPassUsu write SetMenuChgPassUsu;
      property ForcePassword : Boolean read FForcePass write FForcePass;
      property MinPasswordLength  : Integer read FPassLen write FPassLen;
  end;


  TUCAutoLogin = class(TPersistent) // armazenar configuracao de Auto-Logon
    private
      FActive : Boolean;
      FAutoUser, FAutoPassword : String;
      FMessageOnError: Boolean;
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property Active : Boolean read FActive write FActive;
      property User : String read FAutoUser write FAutoUser;
      property Password : String read FAutoPassword write FAutoPassword;
      property MessageOnError : Boolean read FMessageOnError write FMessageOnError;
  end;

  TInitialLogin = class(TPersistent) // armazenar Dados do Login que sera criado na primeira execucao do programa.
    private
      FIniUser: String;
      FIniPass: String;
      FPermIni: TStrings;
      FEmail: String;
      procedure SetFPermIni(const Value: TStrings);
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property User : String read FIniUser write FIniUser;
      property Email : String read FEmail write FEmail;
      property Password : String read FIniPass write FIniPass;
      property InitialRights :  TStrings read FPermIni write SetFPermIni;
    end;
  TGetLoginName = (lnNone, lnUserName, lnMachineName);
  TLogin = class(TPersistent)
    private
      FAutoLogin: TUCAutoLogin;
      FPassNTry: Integer;
      FLoginInicial: TInitialLogin;
    FGetLoginName: TGetLoginName;
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property AutoLogon : TUCAutoLogin read FAutoLogin write FAutoLogin;
      property InitialLogin : TInitialLogin read FLoginInicial write FLoginInicial;
      property MaxLoginAttempts : Integer read FPassNTry write FPassNTry;
      property GetLoginName : TGetLoginName read FGetLoginName write FGetLoginName;
  end;


  TNaoPermitidos = class(TPersistent) // Ocultar e/ou Desabilitar os itens que o usuario nao tem acesso
    private
      FMenuVisible, FActionVisible : Boolean;
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property MenuVisible : Boolean read FMenuVisible write FMenuVisible;
      property ActionVisible : Boolean read FActionVisible write FActionVisible;
  end;

  TLogControl = class(TPersistent) // Responsavel pelo Controle de Log
    private
      FLogActive: Boolean;
      FTableLog: String;
      FMenuLog: TMenuItem;
      FActLog: TAction;
      procedure SetFMenuLog(const Value: TMenuItem);
      procedure SetFActLog(const Value: TAction);
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property Action : TAction read FActLog write SetFActLog;
      property Active : Boolean read FLogActive write FLogActive;
      property TableLog : String read FTableLog write FTableLog;
      property MenuItem : TMenuItem read FMenuLog write SetFMenuLog;
  end;


  TUCActions = class(TPersistent) // no caso de utilizacao de ActionList/ActionManager ou ActionMainMenuBar
    private
      FActionList: TActionList;
      {$IFDEF VER150}
      FActionManager: TActionManager;
      FActionMainMenuBar : TActionMainMenuBar;
      {$ENDIF}
      procedure SetActionList(const Value: TActionList);
      {$IFDEF VER150}
      procedure SetActionManager(const Value: TActionManager);
      procedure  SetActionMainMenuBar(const Value: TActionMainMenuBar);
      {$ENDIF}
    public
      constructor Create(AOwner : TComponent);
      destructor Destroy; override;
      procedure Assign(Source : TPersistent);override;
    published
      property ActionList : TActionList read FActionList write SetActionList;
      {$IFDEF VER150}
      property ActionManager : TActionManager read FActionManager write SetActionManager;
      property ActionMainMenuBar : TActionMainMenuBar read FActionMainMenuBar write SetActionMainMenuBar;
      {$ENDIF}

    end;


  TOnLogin = procedure (Sender : TObject; var User, Password : String) of object;
  TOnLoginSucess = procedure (Sender : TObject; IdUser : integer; Usuario, Nome, Senha, Email : String;
                              Privilegiado : Boolean) of object;
  TOnLoginError = procedure (Sender : TObject; Usuario, Senha : String) of object;
  TOnApplyRightsMenuIt = procedure (Sender : TObject; MenuItem : TMenuItem) of object;
  TOnApllyRightsActionIt = procedure (Sender : TObject; Action : TAction) of Object;
  TCustomCadUsuarioForm = procedure (Sender : TObject; var CustomForm : TCustomForm) of Object;
  TCustomPerfilUsuarioForm = procedure (Sender : TObject; var CustomForm : TCustomForm) of Object;
  TCustomLoginForm = procedure (Sender : TObject; var CustomForm : TCustomForm) of Object;
  TCustomTrocarSenhaForm = procedure (Sender : TObject; var CustomForm : TCustomForm) of Object;
  TCustomLogControlForm = procedure (Sender : TObject; var CustomForm : TCustomForm) of Object;
  TCustomInicialMsg = procedure (Sender : TObject; var CustomForm : TCustomForm; var Msg : TStrings) of Object;
  TOnAddUser = procedure (Sender : TObject; var Login, Password, Name, Mail : String; var Profile : Integer; var Privuser : Boolean) of Object;
  TOnChangeUser = procedure (Sender : TObject; IDUser: Integer; var Login, Name, Mail : String; var Profile : Integer; var Privuser : Boolean) of Object;
  TOnDeleteUser = procedure (Sender : TObject; IDUser: Integer; var CanDelete : Boolean; var ErrorMsg : String) of Object;
  TOnAddProfile = procedure (Sender : TObject; var Profile : String) of Object;
  TOnDeleteProfile = procedure (Sender : TObject; IDProfile : Integer; var CanDelete : Boolean; var ErrorMsg : String) of Object;
  TOnChangePassword = procedure (Sender : TObject; IDUser : Integer; Login, CurrentPassword, NewPassword : String) of Object;
  TOnLogoff = procedure (Sender : TObject; IDUser : integer) of Object;

  TUCSettings = class;

  TUCAboutVar=String[10];

  TUCAboutVarProperty = class;

  TUCDefine = class(TComponent) // Classe principal
  private
    FUser : TUserDef;
    FUserSettings : TUserSettings;
    FAplID : String;
    FNaoPermitidos : TNaoPermitidos;
    FOnLogin: TOnLogin;
    FOnStartApp: TNotifyEvent;
    FOnLoginError: TOnLoginError;
    FOnLoginSucess: TOnLoginSucess;
    FOnApplyRightsActionIt: TOnApllyRightsActionIt;
    FOnApplyRightsMenuIt: TOnApplyRightsMenuIt;
    FLogControl: TLogControl;
{$IFDEF VER130}
{$ELSE}
    FMailUserControl: TMailUserControl;
{$ENDIF}
    FEncrytKey: Word;
    FCadUsuarios: TCadastroUsuarios;
    FLogin: TLogin;
    FPerfUsuarios: TPerfilUsuarios;
    FTrocarSenha: TTrocarSenha;
    FUCActions: TUCActions;
    FOnCustomCadUsuarioForm: TCustomCadUsuarioForm;
    FCustomLogControlForm: TCustomLogControlForm;
    FCustomLoginForm: TCustomLoginForm;
    FCustomPerfilUsuarioForm: TCustomPerfilUsuarioForm;
    FCustomTrocarSenhaForm: TCustomTrocarSenhaForm;
    FOnAddProfile: TOnAddProfile;
    FOnAddUser: TOnAddUser;
    FOnChangePassword: TOnChangePassword;
    FOnChangeUser: TOnChangeUser;
    FOnDeleteProfile: TOnDeleteProfile;
    FOnDeleteUser: TOnDeleteUser;
    FOnLogoff: TOnLogoff;
    FCustomInicialMsg: TCustomInicialMsg;
    FAbout: TUCAboutVar;
    procedure SetWindow;
    procedure SetWindowProfile;
{$IFDEF VER130}
{$ELSE}
    procedure SetFMailUserControl(const Value: TMailUserControl);
{$ENDIF}
    procedure ActionCadUser(Sender: TObject);
    procedure ActionTrocaSenha(Sender: TObject);
    procedure ActionUserProfile(Sender: TObject);
    procedure ActionOKLogin(Sender: TObject);
    procedure TestaFecha(Sender: TObject; var CanClose: Boolean);
    procedure ApplySettings(SourceSettings: TUCSettings);
{$IFDEF VER150}
    procedure TrataActMenuBarIt(IT: TActionClientItem; FDataset : TDataset);
    procedure IncPermissActMenuBar(Act: TAction; idUser : Integer);
{$ENDIF}
  protected
    FRetry : Integer;
    FTabelaUsuarios, FTableRights : String;
    FormCadastroUsuarios, FormPerfilUsuarios, FormTrocarSenha, FormLogin, FormLogControl : TCustomForm;
    procedure Loaded; override;
    procedure CriaCadUserForm; dynamic;
    procedure CriaProfileUserForm; dynamic;
    procedure ActionLog(Sender : TObject); dynamic;
    procedure ActionBtPermiss(Sender : TObject); dynamic;
    procedure ActionBtPermissProfile(Sender: TObject); dynamic;
    procedure CriaTrocaSenhaForm; dynamic;
    procedure ActionTSBtGrava(Sender : TObject);
    procedure SetFUserSettings ( const Value : TUserSettings);
    procedure SetLoginWindow ( Form : TLoginWindow);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RegistraCurrentUser(dados: TDataset);
    procedure UCExecSQL( FSQL : String);virtual;
    procedure ApplyRightsObj(FDataset: TDataset ; FProfile : Boolean = False);
{$IFDEF VER130}
{$ELSE}
    procedure ActionEsqueceuSenha(Sender: TObject);
{$ENDIF}
    function UCGetSQLDataset(FSQL: String): TDataset;dynamic;
    procedure ShowLogin;
    procedure ApplyRights;
    procedure IncPermiss(ItemPerm : TObject; idUser : Integer);
    procedure CriaTableLog;
    procedure CriaTableRights;
    procedure CriaTabelaUsuarios(TableExists : Boolean);
    procedure CriaTabelaMsgs(const TableName : String);
    procedure TryAutoLogon;
    function UCFindTable(const Tablename : String) : Boolean; virtual; abstract;
    function UCFindDataConnection : Boolean; virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    property CurrentUser : TUserDef read FUser write FUser;
    property Settings : TUserSettings read FUserSettings write SetFUserSettings;
    procedure Log( MSG : String; Nivel : Integer = 1);
    function VerificaLogin(User , Password : String) : Boolean;
    procedure Logoff;
    function AddUser(Login, Password, Name, Mail : String; Profile : Integer; PrivUser : Boolean) : Integer;
    procedure ChangeUser(IDUser : Integer; Login, Name, Mail : String; Profile : Integer; PrivUser : Boolean);
    procedure ChangePassword(IDUser : Integer; NewPassword: String);
  published
    property About: TUCAboutVar read FAbout write FAbout;
    property ApplicationID : String read FAplID write FAplID;
    property Actions : TUCActions read FUCActions write FUCActions;
    property UsersForm : TCadastroUsuarios read FCadUsuarios write FCadUsuarios;
    property EncryptKey : word read FEncrytKey write FEncrytKey;
    property NotAllowedItems : TNaoPermitidos read FNaoPermitidos write FNaoPermitidos;
    property Login : TLogin read FLogin write FLogin;
    property LogControl : TLogControl read FLogControl write FLogControl;

{$IFDEF VER130}
{$ELSE}
    property MailUserControl : TMailUserControl read FMailUserControl write SetFMailUserControl;
{$ENDIF}

    property UsersProfile : TPerfilUsuarios read FPerfUsuarios write FPerfUsuarios;
    //property Settings : TUserSettings read FUserSettings write SetFUserSettings;
    property TableUsers : String read FTabelaUsuarios write FTabelaUsuarios;
    property TableRights : String read FTableRights write FTableRights;
    property ChangePasswordForm : TTrocarSenha read FTrocarSenha write FTrocarSenha;

    property OnLogin : TOnLogin read FOnLogin Write FOnLogin;
    property OnStartApplication : TNotifyEvent read FOnStartApp write FOnStartApp;
    property OnLoginSucess : TOnLoginSucess read FOnLoginSucess write FOnLoginSucess;
    property OnLoginError : TOnLoginError read FOnLoginError write FOnLoginError;
    property OnApplyRightsMenuIt : TOnApplyRightsMenuIt read FOnApplyRightsMenuIt write FOnApplyRightsMenuIt;
    property OnApplyRightsActionIt : TOnApllyRightsActionIt read FOnApplyRightsActionIt write FOnApplyRightsActionIt;
    property OnCustomUsersForm : TCustomCadUsuarioForm read FOnCustomCadUsuarioForm write FOnCustomCadUsuarioForm;
    property OnCustomUsersProfileForm : TCustomPerfilUsuarioForm read FCustomPerfilUsuarioForm write FCustomPerfilUsuarioForm;
    property OnCustomLoginForm : TCustomLoginForm read FCustomLoginForm write FCustomLoginForm;
    property OnCustomChangePasswordForm : TCustomTrocarSenhaForm read FCustomTrocarSenhaForm write FCustomTrocarSenhaForm;
    property OnCustomLogControlForm : TCustomLogControlForm read FCustomLogControlForm write FCustomLogControlForm;
    property OnCustomInitialMsg : TCustomInicialMsg read FCustomInicialMsg write FCustomInicialMsg;
    property OnAddUser : TOnAddUser read FOnAddUser write FOnAddUser;
    property OnChangeUser : TOnChangeUser read FOnChangeUser write FOnChangeUser;
    property OnDeleteUser : TOnDeleteUser read FOnDeleteUser write FOnDeleteUser;
    property OnAddProfile : TOnAddProfile read FOnAddProfile write FOnAddProfile;
    property OnDeleteProfile : TOnDeleteProfile read FOnDeleteProfile write FOnDeleteProfile;
    property OnChangePassword : TOnChangePassword read FOnChangePassword write FOnChangePassword;
    property OnLogoff : TOnLogoff read FOnLogoff write FOnLogoff;
  end;

 TVerifThread = class(TThread)
  private
    procedure VerNovaMsg;
  public
    AOwner : TComponent;
  protected
    procedure Execute; override;
  end;

  TUCAppMessage = class(TComponent)
  private
    FActive: Boolean;
    FReady : Boolean;
    FInterval: integer;
    FUserControl: TUCDefine;
    FVerifThread : TVerifThread;
    FTabelaMsg: String;
    procedure SetActive(const Value: Boolean);
  protected
    procedure Loaded; override;
  public
    constructor Create(AOWner : TComponent);override;
    destructor Destroy;override;
    procedure ShowMessages(Modal : Boolean = True);
    procedure SendAppMessage(ToUser: Integer; Subject, Msg: String);
    procedure DeleteAppMessage(IdMsg : Integer);
    procedure CheckMessages;
  published
    property Active : Boolean read FActive write SetActive;
    property Interval : integer read FInterval write FInterval;
    property TableMessages : String read FTabelaMsg write FTabelaMsg;
    property UserControl : TUCDefine read FUserControl write FUserControl;
  end;

  TUCSettings = class(TComponent)
  private
    FAddProfileFormMSG: TAddProfileFormMSG;
    FAddUserFormMSG: TAddUserFormMSG;
    FCadUserFormMSG: TCadUserFormMSG;
    FLogControlFormMSG: TLogControlFormMSG;
    FLoginFormMSG: TLoginFormMSG;
    FPermissFormMSG: TPermissFormMSG;
    FProfileUserFormMSG: TProfileUserFormMSG;
    FResetPassword: TResetPassword;
    FTrocaSenhaFormMSG: TTrocaSenhaFormMSG;
    FUserCommomMSG: TUserCommonMSG;
    FAppMessagesMSG: TAppMessagesMSG;
    procedure SetFAddProfileFormMSG(const Value: TAddProfileFormMSG);
    procedure SetFAddUserFormMSG(const Value: TAddUserFormMSG);
    procedure SetFCadUserFormMSG(const Value: TCadUserFormMSG);
    procedure SetFFormLoginMsg(const Value: TLoginFormMSG);
    procedure SetFLogControlFormMSG(const Value: TLogControlFormMSG);
    procedure SetFPermissFormMSG(const Value: TPermissFormMSG);
    procedure SetFProfileUserFormMSG(const Value: TProfileUserFormMSG);
    procedure SetFResetPassword(const Value: TResetPassword);
    procedure SetFTrocaSenhaFormMSG(const Value: TTrocaSenhaFormMSG);
    procedure SetFUserCommonMSg(const Value: TUserCommonMSG);
    procedure SetAppMessagesMSG(const Value: TAppMessagesMSG);
  protected
  public
    constructor Create(AOwner : TComponent);override;
    destructor Destroy; override;
    procedure Assign(Source : TPersistent); override;
  published
    property AppMessages : TAppMessagesMSG read FAppMessagesMSG write SetAppMessagesMSG;
    property CommonMessages : TUserCommonMSG read FUserCommomMSG write SetFUserCommonMSg;
    property Login : TLoginFormMSG      read FLoginFormMSG      write SetFFormLoginMsg;
    property Log :   TLogControlFormMSG read FLogControlFormMSG write SetFLogControlFormMSG;
    property UsersForm : TCadUserFormMSG read FCadUserFormMSG write SetFCadUserFormMSG;
    property AddChangeUser : TAddUserFormMSG read FAddUserFormMSG write SetFAddUserFormMSG;
    property AddChangeProfile : TAddProfileFormMSG read FAddProfileFormMSG write SetFAddProfileFormMSG;
    property UsersProfile : TProfileUserFormMSG read FProfileUserFormMSG write SetFProfileUserFormMSG;
    property Rights : TPermissFormMSG read FPermissFormMSG write SetFPermissFormMSG;
    property ChangePassword : TTrocaSenhaFormMSG read FTrocaSenhaFormMSG write SetFTrocaSenhaFormMSG;
    property ResetPassword : TResetPassword read FResetPassword write SetFResetPassword;
  end;

  TUCAboutVarProperty = class( TStringProperty )
     function GetAttributes: TPropertyAttributes; override;
     procedure Edit; override;
     function GetValue:String;override;
  end;

procedure Register;
procedure IniSettings( DestSettings : TUserSettings);
procedure IniSettings2(DestSettings: TUCSettings);

implementation

{$R UCLock.res}

{ TUCDefine }
uses
  Dialogs, CadPerfil_U, CadUser_U, UserPermis_U, TrocaSenha_U,
  MsgRecForm_U, MsgsForm_U, UCAbout;


constructor TUCDefine.Create(AOwner: TComponent);
begin
  inherited;
  FUser := TUserDef.Create(self);
  Actions := TUCActions.Create(Self);
  Login := TLogin.Create(Self);
  LogControl := TLogControl.Create(Self);
  UsersForm := TCadastroUsuarios.Create(Self);
  UsersProfile := TPerfilUsuarios.Create(Self);
  ChangePasswordForm := TTrocarSenha.Create(Self);
  FUserSettings := TUserSettings.Create(Self);
  FNaoPermitidos := TNaoPermitidos.Create(Self);
  if csDesigning in ComponentState then
  begin
    if TableUsers = '' then TableUsers := 'UCCadUser';
    if TableRights = '' then TableRights := 'UCCadPerm';
    if LogControl.TableLog = '' then LogControl.TableLog := 'UCLog';
    if ApplicationID = '' then ApplicationID := 'NewProject';
    if Login.InitialLogin.User = '' then Login.InitialLogin.User := 'Admin';
    if Login.InitialLogin.Password = '' then Login.InitialLogin.Password := '#delphi';
    if Login.InitialLogin.Email = '' then Login.InitialLogin.Email := 'qmd@onda.com.br';
    UsersProfile.Active := True;
    UsersForm.UsePrivilegedField := False;
    UsersForm.ProtectAdmin := True;
    NotAllowedItems.MenuVisible := True;
    NotAllowedItems.ActionVisible := True;
  end;
  IniSettings(Settings);

end;


procedure TUCDefine.Loaded;
var
  Contador : integer;
begin
  inherited;
  if not (csDesigning in ComponentState) then
  begin
    for Contador := 0 to Pred(Owner.ComponentCount) do
      if Owner.Components[Contador] is TUCSettings then ApplySettings(TUCSettings(Owner.Components[Contador]));
    if (not Assigned(UsersForm.MenuItem)) and (not Assigned(UsersForm.Action)) then raise Exception.Create(MsgExceptUserMngMenu)
    else begin
      if Assigned(UsersForm.MenuItem) then UsersForm.MenuItem.OnClick := ActionCadUser;
      if Assigned(UsersForm.Action) then UsersForm.Action.OnExecute := ActionCadUser;
    end;

    if UsersProfile.Active then
      if (not Assigned(UsersProfile.MenuItem)) and (not Assigned(UsersProfile.Action)) then raise Exception.Create(MsgExceptUserProfile)
      else
      begin
        if Assigned(UsersProfile.MenuItem) then UsersProfile.MenuItem.OnClick := ActionUserProfile;
        if Assigned(UsersProfile.Action) then UsersProfile.Action.OnExecute := ActionUserProfile;
      end;

    if (not Assigned(ChangePasswordForm.MenuItem) ) and (not Assigned(ChangePasswordForm.Action)) then raise Exception.Create(MsgExceptChagePassMenu)
    else
    begin
      if Assigned(ChangePasswordForm.MenuItem) then ChangePasswordForm.MenuItem.OnClick := ActionTrocaSenha;
      if Assigned(ChangePasswordForm.Action) then ChangePasswordForm.Action.OnExecute := ActionTrocaSenha;
    end;

    if (LogControl.Active) then
      if ( not Assigned(LogControl.MenuItem)) and (not Assigned(LogControl.Action)) then raise Exception.Create('Informe o menu/action do Log')
      else
      begin
        if Assigned(LogControl.MenuItem) then LogControl.MenuItem.OnClick := ActionLog;
        if Assigned(LogControl.Action) then LogControl.Action.OnExecute := ActionLog;
      end;

    if ApplicationID = '' then raise Exception.Create(MsgExceptAppID);
    if TableUsers = '' then raise Exception.Create(MsgExceptUsersTable);
    if TableRights = '' then raise Exception.Create(MsgExceptRightsTable);
    if Assigned(OnStartApplication) then OnStartApplication(self);

    if not UCFindDataConnection then raise Exception.Create(Format(msgExceptConnection,[name]));
    if not UCFindTable(FTableRights) then CriaTableRights;
    if LogControl.Active then
      if not UCFindTable(LogControl.TableLog) then CriaTableLog;

    CriaTabelaUsuarios(UCFindTable(FTabelaUsuarios));
//    if not UCFindTable(FTabelaUsuarios) then CriaTabelaUsuarios(True);

    if not Login.AutoLogon.Active then ShowLogin else TryAutoLogon;
    ApplyRights;

  end;
end;

procedure TUCDefine.ActionUserProfile(Sender : TObject);
begin
  if Assigned(OnCustomUsersProfileForm) then OnCustomUsersProfileForm(Self, FormPerfilUsuarios);
  if FormPerfilUsuarios = nil then CriaProfileUserForm;
  FormPerfilUsuarios.ShowModal;
  FreeAndNil(FormPerfilUsuarios);
end;

procedure TUCDefine.CriaProfileUserForm;
begin
  FormPerfilUsuarios := TCadPerfil.Create(self);
  with Settings.UsersProfile do
  begin
    TCadPerfil(FormPerfilUsuarios).Caption := WindowCaption;
    TCadPerfil(FormPerfilUsuarios).lbDescricao.Caption := LabelDescription;
    TCadPerfil(FormPerfilUsuarios).btAdic.Caption := BtAdd;
    TCadPerfil(FormPerfilUsuarios).BtAlt.Caption := BtChange;
    TCadPerfil(FormPerfilUsuarios).BtExclui.Caption := BtDelete;
    TCadPerfil(FormPerfilUsuarios).BtAcess.Caption := BtRights;   //BGM
    TCadPerfil(FormPerfilUsuarios).BtExit.Caption := BtClose;
    TCadPerfil(FormPerfilUsuarios).BtAcess.OnClick := ActionBtPermissProfile;
  end;
end;


procedure TUCDefine.ActionBtPermissProfile(Sender: TObject);
begin
  if TCadPerfil(FormPerfilUsuarios).DSPerfilUser.RecordCount = 0 then Exit;
  UserPermis := TUserPermis.Create(self);
  SetWindowProfile;
  UserPermis.lbUser.Caption := TCadPerfil(FormPerfilUsuarios).DSPerfilUser.FieldByName('Nome').asString;
end;

procedure TUCDefine.SetWindowProfile;
begin
  with Settings.Rights do
  begin
    UserPermis.Caption := WindowCaption;
    UserPermis.LbDescricao.Caption := LabelProfile;
    UserPermis.PageMenu.Caption := PageMenu;
    UserPermis.PageAction.Caption := PageActions;
    UserPermis.BtLibera.Caption := BtUnlock;
    UserPermis.BtBloqueia.Caption := BtLock;
    UserPermis.BtGrava.Caption := BtSave;
    UserPermis.BtCancel.Caption := BtCancel;
  end;
end;


procedure TUCDefine.ActionCadUser(Sender: TObject);
begin
  if Assigned(OnCustomUsersForm) then OnCustomUsersForm(Self, FormCadastroUsuarios);
  if FormCadastroUsuarios = nil then CriaCadUserForm;
  FormCadastroUsuarios.ShowModal;
  FreeAndNil(FormCadastroUsuarios);
end;


procedure TUCDefine.ActionTrocaSenha(Sender: TObject);
begin
  if Assigned(OnCustomChangePasswordForm) then OnCustomChangePasswordForm(Self, FormTrocarSenha);
  if FormTrocarSenha = nil then CriaTrocaSenhaForm;
  FormTrocarSenha.ShowModal;
  FreeAndNil(FormTrocarSenha);
end;


procedure TUCDefine.CriaCadUserForm;
begin
  FormCadastroUsuarios := TCadUser.Create(self);
  with Settings.UsersForm do
  begin
    TCadUser(FormCadastroUsuarios).Caption := WindowCaption;
    TCadUser(FormCadastroUsuarios).lbDescricao.Caption := LabelDescription;
    TCadUser(FormCadastroUsuarios).btAdic.Caption := BtAdd;
    TCadUser(FormCadastroUsuarios).BtAlt.Caption := BtChange;
    TCadUser(FormCadastroUsuarios).BtExclui.Caption := BtDelete;
    TCadUser(FormCadastroUsuarios).BtAcess.Caption := BtRights;
    TCadUser(FormCadastroUsuarios).BtPass.Caption := BtPassword;
    TCadUser(FormCadastroUsuarios).BtExit.Caption := BtClose;
  end;
  TCadUser(FormCadastroUsuarios).BtAcess.OnClick := ActionBtPermiss;
end;


procedure TUCDefine.ActionBtPermiss(Sender: TObject);
begin
  if TCadUser(FormCadastroUsuarios).DSCadUser.RecordCount = 0 then Exit;
  UserPermis := TUserPermis.Create(self);
  SetWindow;
  UserPermis.lbUser.Caption := TCadUser(FormCadastroUsuarios).DSCadUser.FieldByName('Login').asString;
end;
procedure TUCDefine.SetWindow;
begin
  with Settings.Rights do
  begin
    UserPermis.Caption := WindowCaption;
    UserPermis.LbDescricao.Caption := LabelUser;
    UserPermis.PageMenu.Caption := PageMenu;
    UserPermis.PageAction.Caption := PageActions;
    UserPermis.BtLibera.Caption := BtUnlock;
    UserPermis.BtBloqueia.Caption := BtLOck;
    UserPermis.BtGrava.Caption := BtSave;
    UserPermis.BtCancel.Caption := BtCancel;
  end;
end;

procedure TUCDefine.CriaTrocaSenhaForm;
begin
 FormTrocarSenha := TTrocaSenha.Create(Self);
 with Settings.ChangePassword do begin
   TTrocaSenha(FormTrocarSenha).Caption := WindowCaption;
   TTrocaSenha(FormTrocarSenha).lbDescricao.Caption := LabelDescription;
   TTrocaSenha(FormTrocarSenha).lbSenhaAtu.Caption := LabelCurrentPassword;
   TTrocaSenha(FormTrocarSenha).lbNovaSenha.Caption := LabelNewPassword;
   TTrocaSenha(FormTrocarSenha).lbConfirma.Caption := LabelConfirm;
   TTrocaSenha(FormTrocarSenha).btGrava.Caption := BtSave;
   TTrocaSenha(FormTrocarSenha).btCancel.Caption := BtCancel;
 end;
 TTrocaSenha(FormTrocarSenha).btGrava.OnClick := ActionTSBtGrava;
 if CurrentUser.Password = '' then TTrocaSenha(FormTrocarSenha).EditAtu.Enabled := False;
end;

procedure TUCDefine.ActionTSBtGrava(Sender: TObject);
begin
   if CurrentUser.Password <> TTrocaSenha(FormTrocarSenha).EditAtu.text then
   begin
     MessageDlg(Settings.CommonMessages.ChangePasswordError.InvalidCurrentPassword, mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditAtu.SetFocus;
     Exit;
   end;
   if TTrocaSenha(FormTrocarSenha).EditNova.Text <> TTrocaSenha(FormTrocarSenha).EditConfirma.Text then
   begin
     MessageDlg(Settings.CommonMessages.ChangePasswordError.InvalidNewPassword, mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditNova.SetFocus;
     Exit;
   end;
   if TTrocaSenha(FormTrocarSenha).EditNova.Text = CurrentUser.Password then
   begin
     MessageDlg(Settings.CommonMessages.ChangePasswordError.NewEqualCurrent, mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditNova.SetFocus;
     Exit;
   end;
   if (ChangePasswordForm.ForcePassword) and (TTrocaSenha(FormTrocarSenha).EditNova.Text = '') then
   begin
     MessageDlg( Settings.CommonMessages.ChangePasswordError.PasswordRequired, mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditNova.Text;
     Exit;
   end;
   if Length(TTrocaSenha(FormTrocarSenha).EditNova.Text) < ChangePasswordForm.MinPasswordLength then
   begin
     MessageDlg(Format(Settings.CommonMessages.ChangePasswordError.MinPasswordLength,[ChangePasswordForm.MinPasswordLength]), mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditNova.SetFocus;
     Exit;
   end;
   if Pos(LowerCase(TTrocaSenha(FormTrocarSenha).EditNova.Text),'asdfqwerzxcv1234567890321654987teste'+LowerCase(CurrentUser.Username) + LowerCase(CurrentUser.LoginName)) > 0 then
   begin
     MessageDlg( Settings.CommonMessages.ChangePasswordError.InvalidNewPassword, mtWarning, [mbOK], 0);
     TTrocaSenha(FormTrocarSenha).EditNova.SetFocus;
     Exit;
   end;

   if Assigned( OnChangePassword) then OnChangePassword(Self, CurrentUser.UserID, CurrentUser.LoginName, CurrentUser.Password,TTrocaSenha(FormTrocarSenha).EditNova.Text);
   UCExecSQL( 'Update ' + TableUsers + ' Set Senha = ' + QuotedStr( Encrypt(TTrocaSenha(FormTrocarSenha).EditNova.Text, EncryptKey)) + ' Where IdUser = ' + IntToStr(CurrentUser.UserID) );
   CurrentUser.Password := TTrocaSenha(FormTrocarSenha).EditNova.Text;
   if CurrentUser.Password = '' then MessageDlg(Format(Settings.CommonMessages.BlankPassword,[CurrentUser.LoginName]) , mtInformation, [mbOK], 0)
   else MessageDlg(Settings.CommonMessages.PasswordChanged, mtInformation, [mbOK], 0);

{$IFDEF VER130}
{$ELSE}
   if ( Assigned(MailUserControl) ) and (MailUserControl.SenhaTrocada.Ativo) then
   begin
     with CurrentUser do
     begin
       try
         MailUserControl.EnviaEmailSenhaTrocada( Username, LoginName, TTrocaSenha(FormTrocarSenha).EditNova.Text, Email, '', EncryptKey);
       except
         on e : exception do Log(e.Message,2);
       end;
     end;
   end;
{$ENDIF}
   TTrocaSenha(FormTrocarSenha).Close;
end;


const
  Codes64 = '0A1B2C3D4E5F6G7H89IjKlMnOPqRsTuVWXyZabcdefghijkLmNopQrStUvwxYz+/';
  C1 = 52845;
  C2 = 22719;


function Decode(const S: AnsiString): AnsiString;
const
  Map: array[Char] of Byte = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 62, 0, 0, 0, 63, 52, 53,
    54, 55, 56, 57, 58, 59, 60, 61, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2,
    3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
    20, 21, 22, 23, 24, 25, 0, 0, 0, 0, 0, 0, 26, 27, 28, 29, 30,
    31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 51, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0);
var
  I: LongInt;
begin
  case Length(S) of
    2:
      begin
        I := Map[S[1]] + (Map[S[2]] shl 6);
        SetLength(Result, 1);
        Move(I, Result[1], Length(Result))
      end;
    3:
      begin
        I := Map[S[1]] + (Map[S[2]] shl 6) + (Map[S[3]] shl 12);
        SetLength(Result, 2);
        Move(I, Result[1], Length(Result))
      end;
    4:
      begin
        I := Map[S[1]] + (Map[S[2]] shl 6) + (Map[S[3]] shl 12) +
          (Map[S[4]] shl 18);
        SetLength(Result, 3);
        Move(I, Result[1], Length(Result))
      end
  end
end;

function PreProcess(const S: AnsiString): AnsiString;
var
  SS: AnsiString;
begin
  SS := S;
  Result := '';
  while SS <> '' do
  begin
    Result := Result + Decode(Copy(SS, 1, 4));
    Delete(SS, 1, 4)
  end
end;

function InternalDecrypt(const S: AnsiString; Key: Word): AnsiString;
var
  I: Word;
  Seed: Word;
begin
  Result := S;
  Seed := Key;
  for I := 1 to Length(Result) do
  begin
    Result[I] := Char(Byte(Result[I]) xor (Seed shr 8));
    Seed := (Byte(S[I]) + Seed) * Word(C1) + Word(C2)
  end
end;

function Decrypt(const S: AnsiString; Key: Word): AnsiString;
begin
  Result := InternalDecrypt(PreProcess(S), Key)
end;

function Encode(const S: AnsiString): AnsiString;
const
  Map: array[0..63] of Char = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
    'abcdefghijklmnopqrstuvwxyz0123456789+/';
var
  I: LongInt;
begin
  I := 0;
  Move(S[1], I, Length(S));
  case Length(S) of
    1:
      Result := Map[I mod 64] + Map[(I shr 6) mod 64];
    2:
      Result := Map[I mod 64] + Map[(I shr 6) mod 64] +
        Map[(I shr 12) mod 64];
    3:
      Result := Map[I mod 64] + Map[(I shr 6) mod 64] +
        Map[(I shr 12) mod 64] + Map[(I shr 18) mod 64]
  end
end;

function PostProcess(const S: AnsiString): AnsiString;
var
  SS: AnsiString;
begin
  SS := S;
  Result := '';
  while SS <> '' do
  begin
    Result := Result + Encode(Copy(SS, 1, 3));
    Delete(SS, 1, 3)
  end
end;

function InternalEncrypt(const S: AnsiString; Key: Word): AnsiString;
var
  I: Word;
  Seed: Word;
begin
  Result := S;
  Seed := Key;
  for I := 1 to Length(Result) do
  begin
    Result[I] := Char(Byte(Result[I]) xor (Seed shr 8));
    Seed := (Byte(Result[I]) + Seed) * Word(C1) + Word(C2)
  end
end;

function Encrypt(const S: AnsiString; Key: Word): AnsiString;
begin
  Result := PostProcess(InternalEncrypt(S, Key))
end;

procedure TUCDefine.SetFUserSettings(const Value: TUserSettings);
begin
  Settings := Value;
end;

procedure TUCDefine.SetLoginWindow(Form: TLoginWindow);
begin
  with Settings.Login do
  begin
    Form.Caption := WindowCaption;
    Form.LbUsuario.Caption := LabelUser;
    Form.LbSenha.Caption := LabelPassword;
    Form.btOK.Caption := BtOk;
    Form.BtCancela.Caption := BtCancel;
    if LeftImage <> nil then Form.ImgLeft.Picture.Assign(LeftImage);
    if BottomImage <> nil then Form.ImgBottom.Picture.Assign(BottomImage);
    if TopImage <> nil then Form.ImgTop.Picture.Assign(TopImage);
{$IFDEF VER130}
{$ELSE}
    if Assigned(MailUserControl) then
    begin
      form.lbEsqueci.Visible := MailUserControl.EsqueceuSenha.Ativo;
      form.lbEsqueci.Caption := MailUserControl.EsqueceuSenha.LabelLoginForm;
    end;
{$ENDIF}
  end;
end;


procedure TUCDefine.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) then
  begin
    if AComponent = UsersForm.MenuItem then UsersForm.MenuItem := nil;
    if AComponent = UsersForm.Action then UsersForm.Action := nil;
    if AComponent = UsersProfile.MenuItem then UsersProfile.MenuItem := nil;
    if AComponent = UsersProfile.Action then UsersProfile.Action := nil;
    if AComponent = ChangePasswordForm.Action then ChangePasswordForm.Action := nil;
    if AComponent = ChangePasswordForm.MenuItem then ChangePasswordForm.MenuItem := nil;
    if AComponent = Actions.ActionList then Actions.ActionList := nil;
    {$IFDEF VER150}
    if AComponent = Actions.ActionManager then Actions.ActionManager := nil;
    if AComponent = Actions.ActionMainMenuBar then  Actions.ActionMainMenuBar := nil;
    {$ENDIF}
    if AComponent = LogControl.MenuItem then LogControl.MenuItem := nil;
    if AComponent = LogControl.Action then LogControl.Action := nil;
{$IFDEF VER130}
{$ELSE}
    if AComponent = FMailUserControl then FMailUserControl := nil;
{$ENDIF}
  end;
  inherited Notification (AComponent, Operation);
end;


procedure TUCDefine.ActionLog(Sender: TObject);
begin
  FormLogControl := TViewLog.Create(self);
  with Settings.Log do
  begin
    TViewLog(FormLogControl).Caption := WindowCaption;
    TViewLog(FormLogControl).lbDescricao.Caption := LabelDescription;
    TViewLog(FormLogControl).lbUsuario.Caption := LabelUser;
    TViewLog(FormLogControl).lbData.Caption := LabelDate;
    TViewLog(FormLogControl).lbNivel.Caption := LabelLevel;
    TViewLog(FormLogControl).BtFiltro.Caption := BtFilter;
    TViewLog(FormLogControl).BtExclui.Caption := BtDelete;
    TViewLog(FormLogControl).BtFecha.Caption := BtClose;
    TViewLog(FormLogControl).DbGrid1.Columns[0].Title.Caption := ColLevel;
    TViewLog(FormLogControl).DbGrid1.Columns[1].Title.Caption := ColMessage;
    TViewLog(FormLogControl).DbGrid1.Columns[2].Title.Caption := ColUser;
    TViewLog(FormLogControl).DbGrid1.Columns[3].Title.Caption := ColDate;
  end;
end;

procedure TUCDefine.Log(MSG: String; Nivel: Integer);
begin
  if not LogControl.Active then Exit;
  UCExecSQL('Insert into ' + LogControl.TableLog + '( IdUser, MSG, Data, Nivel) Values ( '+
            IntToStr(CurrentUser.UserID)+', '+
            QuotedStr(Copy(MSG,1,250))+', '+
            QuotedStr(FormatDateTime('YYYYMMDDhhmmss',now))+', '+
            IntToStr(Nivel)+')');
end;
{$IFDEF VER130}
{$ELSE}
procedure TUCDefine.SetFMailUserControl(const Value: TMailUserControl);
begin
  FMailUserControl := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;
{$ENDIF}

procedure TUCDefine.RegistraCurrentUser( dados : TDataset);
begin
  with CurrentUser do
  begin
    UserID := Dados.FieldByName('IdUser').asInteger;
    Username := Dados.FieldByName('Nome').asString;
    LoginName:= Dados.FieldByName('Login').asString;
    Password := Decrypt(Dados.FieldByName('Senha').asString, EncryptKey);
    Email := Dados.FieldByName('Email').asString;
    Privilegiado := StrToBool(Dados.FieldByName('Privilegiado').asString);
    Profile := Dados.FieldByName('Perfil').asInteger;
    if Assigned(OnLoginSucess) then OnLoginSucess(Self, UserID, LoginName, UserName, Password, EMail, Privilegiado);
  end;
end;

procedure TUCDefine.UCExecSQL(FSQL: String);
begin
 // to override
end;

{$IFDEF VER130}
{$ELSE}
procedure TUCDefine.ActionEsqueceuSenha(Sender: TObject);
var
  FDataset : TDataset;
begin
  FDataset := UCGetSQLDataset('Select * from '+ TableUsers + ' Where Login = '+ QuotedStr(TLoginWindow(FormLogin).EditUsuario.Text));
  with FDataset do
    try
      if RecordCount > 0 then
        MailUserControl.EnviaEsqueceuSenha( fieldbyname('nome').asString,
                                            fieldbyname('login').asString,
                                            fieldbyname('senha').asString,
                                            fieldbyname('email').asString, '', EncryptKey);
    finally
      Close;
      Free;
    end;
end;
{$ENDIF}
procedure TUCDefine.TryAutoLogon;
begin
  if not VerificaLogin(Login.AutoLogon.User, Login.AutoLogon.Password) then
  begin
    if Login.AutoLogon.MessageOnError then MessageDlg(Settings.CommonMessages.AutoLogonError, mtWarning, [mbOK], 0);
    ShowLogin;
  end;
end;


function TUCDefine.VerificaLogin(User, Password: String): Boolean;
var
  vSenha : String;
  FDataset : TDataset;
begin
  vSenha := 'Senha = ' + QuotedStr(Encrypt(Password, EncryptKey));
  FDataset := UCGetSQLDataset('Select * from '+ TableUsers + ' Where Login = '+ QuotedStr(User) + ' and ' + vSenha);
  with FDataset do
    try
      if RecordCount > 0 then
      begin
        Result := True;
        RegistraCurrentuser(FDataset);
      end else begin
        Result := False;
        if Assigned(OnLoginError) then OnLoginError(Self, User, Password);
      end;
    finally
      Close;
      Free;
    end;
end;

function TUCDefine.UCGetSQLDataset(FSQL: String): TDataset;
begin
  Result := nil;
  //to override
end;

procedure TUCDefine.Logoff;
begin
  if Assigned(onLogoff) then onLogoff(Self, CurrentUser.UserID);
  CurrentUser.UserID := 0;
  ShowLogin;
  ApplyRights;
end;

function TUCDefine.AddUser(Login, Password, Name, Mail: String;
  Profile: Integer; PrivUser: Boolean) : Integer;
begin
  with UCGetSQLDataset('Select Max(IdUser) as IdUser from ' + TableUsers) do
  begin
    Result := Fieldbyname('idUser').asInteger + 1;
    Close;
    Free;
  end;
  UCExecSQL('Insert into ' +  TableUsers + '(IdUser, Nome, Login, Senha, Email, Privilegiado, Perfil, Tipo)'+
            'Values (' + IntToStr(Result)+', '+ QuotedStr(Name)+', '+ QuotedStr(Login)+', '+ QuotedStr(Encrypt(Password, EncryptKey))+', '+QuotedStr(Mail)+', '+ BooltoStr(PrivUser)+', '+IntToStr(Profile)+', '+QuotedStr('U')+')');
  if Assigned(OnAddUser) then OnAddUser(Self, Login, Password, Name, Mail, Profile, Privuser);
end;

procedure TUCDefine.ChangePassword(IDUser: Integer; NewPassword: String);
var
  FLogin, FSenha : String;
begin
  inherited;
  with UCGetSQLDataset('Select Login, Senha from ' + TableUsers+' where IdUser = '+ IntToStr(IdUser)) do
  begin
    FLogin := Fieldbyname('Login').asString;
    FSenha := Decrypt(FieldByName('Senha').asString, EncryptKey);
    FSenha := Copy(FSenha,1,length(FSenha)-4);
    Close;
    Free;
  end;
  UCExecSQL('Update ' +  TableUsers + 'Set Senha = '+ QuotedStr(Encrypt(NewPassword, EncryptKey))+' where IDUser = '+ IntToStr(IdUser));
  if Assigned(onChangePassword) then OnChangePassword(Self, IdUser, FLogin, FSenha, NewPassword);
end;

procedure TUCDefine.ChangeUser(IDUser: Integer; Login, Name,
  Mail: String; Profile: Integer; PrivUser: Boolean);
var
  FLogin, FSenha : String;
begin
  with UCGetSQLDataset('Select Login, Senha from ' + TableUsers+' where IdUser = '+ IntToStr(IdUser)) do
  begin
    FLogin := Fieldbyname('Login').asString;
    FSenha := Decrypt(FieldByName('Senha').asString, EncryptKey);
    FSenha := Copy(FSenha,1,length(FSenha)-4);
    Close;
    Free;
  end;

  UCExecSQL('Update ' +  TableUsers + ' Set '+
  ' Nome = '+ QuotedStr(Name)+', '+
  ' Login = ' + QuotedStr(Login)+', '+
  ' Senha = ' + QuotedStr(Encrypt(FSenha, EncryptKey))+', '+
  ' Email = ' + QuotedStr(Mail)+', '+
  ' Privilegiado = ' +BooltoStr(PrivUser)+', '+
  ' Perfil = ' + IntToStr(Profile)+
  ' where IDUser = '+ IntToStr(IdUser));
  if Assigned(OnChangeUser) then OnChangeUser(Self, IdUser, Login, Name, Mail, Profile, PrivUser);

end;

procedure IniSettings(DestSettings: TUserSettings);
var
  tmp : TBitmap;
begin
    with DestSettings.CommonMessages do
    begin
      if BlankPassword = '' then BlankPassword := Const_Men_SenhaDesabitada;
      if PasswordChanged = '' then PasswordChanged := Const_Men_SenhaAlterada;
      if InitialMessage.Text = '' then InitialMessage.Text := Const_Men_MsgInicial;
      if MaxLoginAttemptsError = '' then MaxLoginAttemptsError := Const_Men_MaxTentativas;
      if InvalidLogin = '' then InvalidLogin := Const_Men_LoginInvalido;
      if AutoLogonError = '' then AutoLogonError := Const_Men_AutoLogonError;
    end;
    with DestSettings.Login do
    begin
      if BtCancel = '' then BtCancel := Const_Log_BtCancelar;
      if BtOK = '' then BtOK := Const_Log_BtOK;
      if LabelPassword = '' then LabelPassword := Const_Log_LabelSenha;
      if LabelUser = '' then LabelUser := Const_Log_LabelUsuario;
      if WindowCaption = '' then WindowCaption := Const_Log_WindowCaption;

      Tmp := TBitmap.create;
      Tmp.LoadFromResourceName(HInstance, 'UCLOCKLOGIN');
      LeftImage.Assign(tmp);

    end;
    with DestSettings.UsersForm do
    begin
      if WindowCaption = '' then WindowCaption := Const_Cad_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Cad_LabelDescricao;
      if ColName = '' then ColName := Const_Cad_ColunaNome;
      if ColLogin = '' then ColLogin := Const_Cad_ColunaLogin;
      if ColEmail = '' then ColEmail := Const_Cad_ColunaEmail;
      if BtAdd = '' then BtAdd := Const_Cad_BtAdicionar;
      if BtChange = '' then BtChange := Const_Cad_BtAlterar;
      if BtDelete = '' then BtDelete := Const_Cad_BtExcluir;
      if BtRights = '' then BtRights := Const_Cad_BtPermissoes;
      if BtPassword = '' then BtPassword := Const_Cad_BtSenha;
      if BtClose = '' then BtClose := Const_Cad_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_Cad_ConfirmaExcluir;
    end;
    with DestSettings.UsersProfile do
    begin
      if WindowCaption = '' then WindowCaption := Const_Prof_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Prof_LabelDescricao;
      if ColProfile = '' then ColProfile := Const_Prof_ColunaNome;
      if BtAdd = '' then BtAdd := Const_Prof_BtAdicionar;
      if BtChange = '' then BtChange := Const_Prof_BtAlterar;
      if BtDelete = '' then BtDelete := Const_Prof_BtExcluir;
      if BtRights = '' then BtRights := Const_Prof_BtPermissoes;    //BGM
      if BtClose = '' then BtClose := Const_Prof_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_Prof_ConfirmaExcluir;
    end;
    with DestSettings.AddChangeUser do
    begin
      if WindowCaption = '' then WindowCaption := Const_Inc_WindowCaption;
      if LabelAdd = '' then LabelAdd := Const_Inc_LabelAdicionar;
      if LabelChange = '' then LabelChange := Const_Inc_LabelAlterar;
      if LabelName = '' then LabelName := Const_Inc_LabelNome;
      if LabelLogin = '' then LabelLogin := Const_Inc_LabelLogin;
      if LabelEmail = '' then LabelEmail := Const_Inc_LabelEmail;
      if CheckPrivileged = '' then CheckPrivileged := Const_Inc_CheckPrivilegiado;
      if BtSave = '' then BtSave := Const_Inc_BtGravar;
      if BtCancel = '' then BtCancel := Const_Inc_BtCancelar;
    end;
    with DestSettings.AddChangeProfile do
    begin
      if WindowCaption = '' then WindowCaption := Const_PInc_WindowCaption;
      if LabelAdd = '' then LabelAdd := Const_PInc_LabelAdicionar;
      if LabelChange = '' then LabelChange := Const_PInc_LabelAlterar;
      if LabelName = '' then LabelName := Const_PInc_LabelNome;
      if BtSave = '' then BtSave := Const_PInc_BtGravar;
      if BtCancel = '' then BtCancel := Const_PInc_BtCancelar;
    end;
    with DestSettings.Rights do
    begin
      if WindowCaption = '' then WindowCaption := Const_Perm_WindowCaption;
      if LabelUser = '' then LabelUser := Const_Perm_LabelUsuario;
      if LabelProfile = '' then LabelProfile := Const_Perm_LabelPerfil;
      if PageMenu = '' then PageMenu := Const_Perm_PageMenu;
      if PageActions = '' then PageActions := Const_Perm_PageActions;
      if BtUnlock = '' then BtUnlock := Const_Perm_BtLibera;
      if BtLock = '' then BtLock := Const_Perm_BtBloqueia;
      if BtSave = '' then BtSave := Const_Perm_BtGravar;
      if BtCancel = '' then BtCancel := Const_Perm_BtCancelar;
    end;
    with DestSettings.ChangePassword do
    begin
      if WindowCaption = '' then WindowCaption := Const_Troc_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Troc_LabelDescricao;
      if LabelCurrentPassword = '' then LabelCurrentPassword := Const_Troc_LabelSenhaAtual;
      if LabelNewPassword = '' then LabelNewPassword := Const_Troc_LabelNovaSenha;
      if LabelConfirm = '' then LabelConfirm := Const_Troc_LabelConfirma;
      if BtSave = '' then BtSave := Const_Troc_BtGravar;
      if BtCancel = '' then BtCancel := Const_Troc_BtCancelar;
    end;
    with DestSettings.CommonMessages.ChangePasswordError do
    begin
      if InvalidCurrentPassword = '' then InvalidCurrentPassword :=  Const_ErrPass_SenhaAtualInvalida;
      if NewPasswordError = '' then NewPasswordError :=  Const_ErrPass_ErroNovaSenha;
      if NewEqualCurrent = '' then NewEqualCurrent :=  Const_ErrPass_NovaIgualAtual;
      if PasswordRequired = '' then PasswordRequired :=  Const_ErrPass_SenhaObrigatoria;
      if MinPasswordLength = '' then MinPasswordLength := Const_ErrPass_SenhaMinima;
      if InvalidNewPassword= '' then InvalidNewPassword :=  Const_ErrPass_SenhaInvalida;
    end;
    with DestSettings.ResetPassword do
    begin
      if WindowCaption = '' then WindowCaption := Const_DefPass_WindowCaption;
      if LabelPassword = '' then LabelPassword := Const_DefPass_LabelSenha;
    end;
    with DestSettings.Log do
    begin
      if WindowCaption = '' then WindowCaption := Const_LogC_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_LogC_LabelDescricao;
      if LabelUser = '' then LabelUser := Const_LogC_LabelUsuario;
      if LabelDate = '' then LabelDate := Const_LogC_LabelData;
      if LabelLevel = '' then LabelLevel := Const_LogC_LabelNivel;
      if ColLevel = '' then ColLevel := Const_LogC_ColunaNivel;
      if ColMessage = '' then ColMessage := Const_LogC_ColunaMensagem;
      if ColUser = '' then ColUser := Const_LogC_ColunaUsuario;
      if ColDate = '' then ColDate := Const_LogC_ColunaData;
      if BtFilter = '' then BtFilter := Const_LogC_BtFiltro;
      if BtDelete = '' then BtDelete := Const_LogC_BtExcluir;
      if BtClose = '' then BtClose := Const_LogC_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_LogC_ConfirmaExcluir;
    end;
    with DestSettings.AppMessages do
    begin
      if MsgsForm_BtNew = '' then MsgsForm_BtNew := Const_Msgs_BtNew;
      if MsgsForm_BtReplay = '' then MsgsForm_BtReplay := Const_Msgs_BtReplay;
      if MsgsForm_BtForward = '' then MsgsForm_BtForward := Const_Msgs_BtForward;
      if MsgsForm_BtDelete = '' then MsgsForm_BtDelete := Const_Msgs_BtDelete;
      if MsgsForm_WindowCaption = '' then MsgsForm_WindowCaption := Const_Msgs_WindowCaption;
      if MsgsForm_ColFrom = '' then  MsgsForm_ColFrom := Const_Msgs_ColFrom;
      if MsgsForm_ColSubject = '' then  MsgsForm_ColSubject := Const_Msgs_ColSubject;
      if MsgsForm_ColDate = '' then MsgsForm_ColDate := Const_Msgs_ColDate;
      if MsgsForm_PromptDelete = '' then  MsgsForm_PromptDelete := Const_Msgs_PromptDelete;

      if MsgRec_BtClose = '' then  MsgRec_BtClose := Const_MsgRec_BtClose;
      if MsgRec_WindowCaption = '' then MsgRec_WindowCaption := Const_MsgRec_WindowCaption;
      if MsgRec_Title = ''then MsgRec_Title := Const_MsgRec_Title;
      if MsgRec_LabelFrom = ''then MsgRec_LabelFrom := Const_MsgRec_LabelFrom;
      if MsgRec_LabelDate = '' then MsgRec_LabelDate := Const_MsgRec_LabelDate;
      if MsgRec_LabelSubject = '' then MsgRec_LabelSubject := Const_MsgRec_LabelSubject;
      if MsgRec_LabelMessage = '' then MsgRec_LabelMessage := Const_MsgRec_LabelMessage;

      if MsgSend_BtSend =  '' then MsgSend_BtSend := Const_MsgSend_BtSend;
      if MsgSend_BtCancel = '' then MsgSend_BtCancel := Const_MsgSend_BtCancel;
      if MsgSend_WindowCaption = '' then MsgSend_WindowCaption := Const_MsgSend_WindowCaption;
      if MsgSend_Title = '' then MsgSend_Title := Const_MsgSend_Title;
      if MsgSend_GroupTo = '' then MsgSend_GroupTo := Const_MsgSend_GroupTo;
      if MsgSend_RadioUser = '' then MsgSend_RadioUser := Const_MsgSend_RadioUser;
      if MsgSend_RadioAll = '' then MsgSend_RadioAll := Const_MsgSend_RadioAll;
      if MsgSend_GroupMessage = '' then MsgSend_GroupMessage := Const_MsgSend_GroupMessage;
    end;
end;


procedure TUCDefine.CriaTabelaMsgs(const TableName : String);
begin
  UCExecSQL('Create Table '+ TableName + ' ( '+
                         'IdMsg int ,'+
                         'UsrFrom int, '+
                         'UsrTo int, '+
                         'Subject Varchar(50),'+
                         'Msg Varchar(255),'+
                         'DtSend Varchar(12),'+
                         'DtReceive Varchar(12) )');
end;

{ TUCAutoLogin }

procedure TUCAutoLogin.Assign(Source: TPersistent);
begin
  if Source is TUCAutoLogin then
  begin
    Self.Active := TUCAutoLogin(Source).Active;
    Self.User := TUCAutoLogin(Source).User;
    Self.Password := TUCAutoLogin(Source).Password;
  end else inherited;
end;

constructor TUCAutoLogin.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TUCAutoLogin.Destroy;
begin
  inherited;
end;

{ TNaoPermitidos }

procedure TNaoPermitidos.Assign(Source: TPersistent);
begin
  if Source is TNaoPermitidos then
  begin
    Self.MenuVisible := TNaoPermitidos(Source).MenuVisible;
    Self.ActionVisible := TNaoPermitidos(Source).FMenuVisible;
  end else inherited;
end;

constructor TNaoPermitidos.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TNaoPermitidos.Destroy;
begin
  inherited;
end;

{ TLogControl }

procedure TLogControl.Assign(Source: TPersistent);
begin
  if Source is TLogControl then
  begin
    Self.Active := TLogControl(Source).Active;
    Self.TableLog := TLogControl(Source).TableLog;
    Self.MenuItem := TLogControl(Source).MenuItem;
    Self.Action := TLogControl(Source).Action;
  end else inherited;
end;

constructor TLogControl.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TLogControl.Destroy;
begin
  inherited;
end;

procedure TLogControl.SetFActLog(const Value: TAction);
begin
  FActLog := Value;
  if Value <> nil then Value.FreeNotification(Self.Action);
end;

procedure TLogControl.SetFMenuLog(const Value: TMenuItem);
begin
  FMenuLog := Value;
  if Value <> nil then Value.FreeNotification(Self.MenuItem);
end;


procedure TUCDefine.ShowLogin;
begin
  FRetry := 0;
  if Assigned(onCustomLoginForm) then OnCustomLoginForm(Self, FormLogin);
  if FormLogin = nil then
  begin
    FormLogin := TLoginWindow.Create(self);
    with FormLogin as TLoginWindow do
    begin
      SetLoginWindow(TLoginWindow(FormLogin));
      btOK.onClick := ActionOKLogin;
      onCloseQuery := Testafecha;
  {$IFDEF VER130}
  {$ELSE}
      lbEsqueci.OnClick := ActionEsqueceuSenha;
  {$ENDIF}
    end;
  end;
  FormLogin.ShowModal;
  FreeAndNil(FormLogin);
end;

procedure TUCDefine.ActionOKLogin(Sender: TObject);
var
  TempUser, TempPassword : String;
begin
  TempUser := TLoginWindow(FormLogin).EditUsuario.Text;
  TempPassword := TLoginWindow(FormLogin).EditSenha.Text;
  if Assigned(OnLogin) then Onlogin(Self, TempUser, TempPassword);
  if VerificaLogin(TempUser, TempPassword) then TLoginWindow(FormLogin).Close
  else
  begin
    MessageDlg(Settings.CommonMessages.InvalidLogin, mtWarning, [mbOK], 0);
    Inc(FRetry);
    if (Login.MaxLoginAttempts > 0) and ( FRetry = Login.MaxLoginAttempts) then
    begin
      MessageDlg(Format(Settings.CommonMessages.MaxLoginAttemptsError,[Login.MaxLoginAttempts]), mtError, [mbOK], 0);
      halt;
    end;
  end;
end;

procedure TUCDefine.TestaFecha(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := (CurrentUser.UserID > 0);
end;


procedure TUCDefine.ApplyRights;
var
  TempDS : TDataset;
begin
  // Aplica Permissoes do Usuario logado
  TempDS := UCGetSQLDataset('Select ObjName from '+ TableRights +  ' Where IdUser = ' +
                        IntToStr(CurrentUser.UserID)+ ' And Modulo = ' + QuotedStr(ApplicationID));
  ApplyRightsObj(TempDS);
  TempDS.Close;


  // Aplica Permissoes do Perfil do usuario
  if CurrentUser.Profile > 0 then
  begin
    TempDS := UCGetSQLDataset('Select ObjName from '+ TableRights +  ' Where IdUser = ' +
                          IntToStr(CurrentUser.Profile)+ ' And Modulo = ' + QuotedStr(ApplicationID));
    ApplyRightsObj(TempDS, True);
    TempDS.Close;
  end;
  FreeAndNil(TempDS);
end;

procedure TUCDefine.ApplyRightsObj( FDataset : TDataset; FProfile : Boolean = False);
var
  Contador : integer;
  Encontrado : Boolean;
  Temp : String;
  ObjAct : TObject;
begin
  //Permissao de Menus
  For Contador := 0 to Owner.ComponentCount-1 do
    if (Owner.Components[Contador].ClassType = TMenuItem) and (TMenuItem(Owner.Components[Contador]).GetParentMenu.ClassType <> TPopupMenu) then begin
      if not FProfile then
      begin
        Encontrado := FDataset.Locate('ObjName', Owner.Components[Contador].Name,[]);
        TMenuItem(Owner.Components[Contador]).Enabled := Encontrado;
        if Not Encontrado then TMenuItem(Owner.Components[Contador]).Visible := NotAllowedItems.MenuVisible
        else TMenuItem(Owner.Components[Contador]).Visible := True;
      end else
      begin
        if FDataset.Locate('ObjName', Owner.Components[Contador].Name,[]) then
        begin
          TMenuItem(Owner.Components[Contador]).Enabled := True;
          TMenuItem(Owner.Components[Contador]).Visible := True;
        end;
      end;
      if Assigned(OnApplyRightsMenuIt) then OnApplyRightsMenuIt(Self, TMenuItem(Owner.Components[Contador]));
    end;

  //Permissao de Actions
  if (Assigned(Actions.ActionList) ) {$IFDEF VER150}or (Assigned(Actions.ActionManager)) or (Assigned(Actions.ActionMainMenuBar)){$ENDIF} then
  begin
    if Assigned(Actions.ActionList) then ObjAct := Actions.ActionList {$IFDEF VER150}else ObjAct := Actions.ActionManager{$ENDIF};
    for contador := 0 to TActionList(ObjAct).ActionCount -1 do
    begin
      if not FProfile then
      begin
        Encontrado := FDataset.Locate('ObjName', TAction(TActionList(ObjAct).Actions[contador]).Name ,[]);
        TAction(TActionList(ObjAct).Actions[contador]).Enabled := Encontrado;
        if not Encontrado then TAction(TActionList(ObjAct).Actions[contador]).Visible := NotAllowedItems.ActionVisible
        else TAction(TActionList(ObjAct).Actions[contador]).Visible := True;
      end else begin
        if FDataset.Locate('ObjName', TAction(TActionList(ObjAct).Actions[contador]).Name ,[]) then
        begin
          TAction(TActionList(ObjAct).Actions[contador]).Enabled := True;
          TAction(TActionList(ObjAct).Actions[contador]).Visible := True;
        end;
      end;
      if Assigned(OnApplyRightsActionIt) then OnApplyRightsActionIt(Self, TAction(TActionList(ObjAct).Actions[contador]));
    end;
  end;
  {$IFDEF VER150}
  if Assigned(Actions.ActionMainMenuBar) then
  begin
    for Contador := 0 to Actions.ActionMainMenuBar.ActionClient.Items.Count-1 do
    begin
      Temp := IntToStr(Contador);
      if Actions.ActionMainMenuBar.ActionClient.Items[StrToInt(Temp)].Items.Count > 0 then
      begin
        Actions.ActionMainMenuBar.ActionClient.Items[StrToInt(Temp)].Visible :=
          (FDataset.Locate('ObjName',#1+'G'+Actions.ActionMainMenuBar.ActionClient.Items[StrToInt(Temp)].Caption,[]));
        TrataActMenuBarIt(Actions.ActionMainMenuBar.ActionClient.Items[StrToInt(Temp)], FDataset);
      end;
    end;
  end;
  {$ENDIF}
end;

{$IFDEF VER150}
procedure TUCDefine.TrataActMenuBarIt(IT : TActionClientItem; FDataset : TDataset);
var
  Contador : integer;
begin
  for contador := 0 to IT.Items.Count -1 do
    if IT.Items[Contador].Caption <> '-' then
      if IT.Items[Contador].Items.Count > 0 then
      begin
        IT.Items[Contador].Visible := (FDataset.Locate('ObjName',#1+'G'+IT.Items[Contador].Caption,[]));
        TrataActMenuBarIt(IT.Items[Contador], FDataset);
      end;
end;
{$ENDIF}
procedure TUCDefine.CriaTableRights;
begin
  UCExecSQL('Create Table '+ FTableRights + ' ( '+
                         'IdUser int, '+
                         'Modulo varchar(50), '+
                         'ObjName varchar(50) )');
end;

procedure TUCDefine.IncPermiss(ItemPerm : TObject; idUser : Integer);
var
  Obj : TObject;
begin
  if ItemPerm = nil then Exit;
  Obj := ItemPerm;
  if Obj.ClassType = TMenuItem then
  begin
    While (Obj.ClassType = TMenuItem) and (TComponent(Obj).Name <> '')do
    begin
      UCExecSQL('Insert into ' + TableRights + '(IdUser, Modulo, ObjName) Values( '+IntToStr(idUser)+', ' + QuotedStr(ApplicationID)+ ', '+ QuotedStr(TComponent(Obj).Name)+')');
      Obj := TMenuItem(Obj).Parent;
    end;
  end
  else
    UCExecSQL('Insert into ' + TableRights + '(IdUser, Modulo, ObjName) Values( '+IntToStr(idUser)+', ' + QuotedStr(ApplicationID)+ ', '+ QuotedStr(TComponent(Obj).Name)+')');
end;


procedure TUCDefine.CriaTableLog;
begin
  UCExecSQL('Create Table '+ LogControl.TableLog + ' ( '+
                         'IdUser int ,'+
                         'MSG varchar(250), '+
                         'Data varchar(14), '+
                         'Nivel Int)');
end;

{$IFDEF VER150}
procedure TUCDefine.IncPermissActMenuBar( Act : TAction; idUser : Integer);
var
  Temp : TActionClientItem;
begin
  if Act = nil then exit;

  Temp := Actions.ActionMainMenuBar.ActionManager.FindItemByAction(Act);
  while Temp <> nil do
  begin
    if Temp.Caption <> '' then UCExecSQL('Insert into ' + TableRights + '(IdUser, Modulo, ObjName) Values( '+IntToStr(idUser)+', ' + QuotedStr(ApplicationID)+ ', '+ QuotedStr(#1+'G'+Temp.Caption)+')');
    Temp := TActionClientItem(TActionClientItem(Temp).ParentItem);
  end;
end;
{$ENDIF}

procedure TUCDefine.CriaTabelaUsuarios(TableExists : Boolean);
var
  Contador, FIdUser : integer;
  FCustomform : TCustomForm;
  FMsg : TStrings;
  DSUser, DSPerm : TDataSet;
begin
  if Not TableExists then
    UCExecSQL('Create Table '+ TableUsers + ' ( '+
                           'IdUser int,'+
                           'Nome varchar(30), '+
                           'Login varchar(15), '+
                           'Senha varchar(30), '+
                           'Email varchar(100), '+
                           'Privilegiado int , '+
                           'Tipo char(1), '+
                           'Perfil int )');
  DSUser := UCGetSQLDataset('Select iduser from '+ TableUsers + ' where login = '+ QuotedStr(Login.InitialLogin.User));
  if (DSUser.RecordCount = 0) then
    FIDUser := AddUser(Login.InitialLogin.User,Login.InitialLogin.Password, Login.InitialLogin.User, Login.InitialLogin.Email,0,True)
  else FIDUser := DSUser.FieldByName('idUser').asInteger;
  DSUser.Close;
  FreeAndNil(DSUser);
  DSPerm := UCGetSQLDataset('Select iduser from '+ TableRights + ' where idUser = '+ IntToStr(FIDUser)+ ' and Modulo = '+QuotedStr(ApplicationID));
  if DSPerm.RecordCount > 0 then
  begin
      DSPerm.Close;
      FreeAndNil(DSPerm);
      Exit; //login!
  end;
  DSPerm.Close;
  FreeAndNil(DSPerm);

  IncPermiss(UsersForm.MenuItem, FIDUser);
  IncPermiss(UsersForm.Action, FIDUser);
  IncPermiss(UsersProfile.MenuItem, FIDUser);
  IncPermiss(UsersProfile.Action, FIDUser);

  IncPermiss(ChangePasswordForm.MenuItem, FIDUser);
  IncPermiss(ChangePasswordForm.Action, FIDUser);

  {$IFDEF Ver150}
  if Assigned(Actions.ActionMainMenuBar) then IncPermissActMenuBar(UsersForm.Action, FIDUser);
  if Assigned(Actions.ActionMainMenuBar) then IncPermissActMenuBar(UsersProfile.Action, FIDUser);
  if Assigned(Actions.ActionMainMenuBar) then IncPermissActMenuBar(ChangePasswordForm.Action, FIDUser);
  {$ENDIF}
  if LogControl.Active then
  begin
    IncPermiss(LogControl.MenuItem, FIDUser);
    IncPermiss(LogControl.Action, FIDUser);
    {$IFDEF Ver150}
    if Assigned(Actions.ActionMainMenuBar) then IncPermissActMenuBar(LogControl.Action, FIDUser);
    {$ENDIF}
  end;

  for contador := 0 to Pred(Login.InitialLogin.InitialRights.Count) do
    if Owner.FindComponent(Login.InitialLogin.InitialRights[contador]) <> nil then  IncPermiss( Owner.FindComponent(Login.InitialLogin.InitialRights[contador]), FIDUser);

  FMsg := TStringList.Create;
  FMsg.Assign(Settings.CommonMessages.InitialMessage);
  if Assigned(OnCustomInitialMsg) then OnCustomInitialMsg(Self,FCustomForm ,FMsg);
  if FCustomForm <> nil then FCustomForm.ShowModal else MessageDlg(FMsg.Text, mtInformation, [mbOK], 0);
end;


procedure TUCDefine.ApplySettings(SourceSettings: TUCSettings);
begin
    with Settings.CommonMessages do
    begin
      BlankPassword := SourceSettings.CommonMessages.BlankPassword;
      PasswordChanged := SourceSettings.CommonMessages.PasswordChanged;
      InitialMessage.Text := SourceSettings.CommonMessages.InitialMessage.Text;
      MaxLoginAttemptsError := SourceSettings.CommonMessages.MaxLoginAttemptsError;
      InvalidLogin := SourceSettings.CommonMessages.InvalidLogin;
      AutoLogonError := SourceSettings.CommonMessages.AutoLogonError;
    end;
    with Settings.Login do
    begin
      BtCancel := SourceSettings.Login.BtCancel;
      BtOK := SourceSettings.Login.BtOK;
      LabelPassword := SourceSettings.Login.LabelPassword;
      LabelUser := SourceSettings.Login.LabelUser;
      WindowCaption := SourceSettings.Login.WindowCaption;
      if Assigned(SourceSettings.Login.LeftImage.Bitmap) then LeftImage.Bitmap := SourceSettings.Login.LeftImage.Bitmap
      else LeftImage.Bitmap := nil;
      if Assigned(SourceSettings.Login.TopImage.Bitmap) then TopImage.Bitmap := SourceSettings.Login.TopImage.Bitmap
      else TopImage.Bitmap := nil;
      if Assigned(SourceSettings.Login.BottomImage.Bitmap) then BottomImage.Bitmap := SourceSettings.Login.BottomImage.Bitmap
      else BottomImage.Bitmap := nil;
    end;
    with Settings.UsersForm do
    begin
      WindowCaption := SourceSettings.UsersForm.WindowCaption;
      LabelDescription := SourceSettings.UsersForm.LabelDescription;
      ColName := SourceSettings.UsersForm.ColName;
      ColLogin := SourceSettings.UsersForm.ColLogin;
      ColEmail := SourceSettings.UsersForm.ColEmail;
      BtAdd := SourceSettings.UsersForm.BtAdd;
      BtChange := SourceSettings.UsersForm.BtChange;
      BtDelete := SourceSettings.UsersForm.BtDelete;
      BtRights := SourceSettings.UsersForm.BtRights;
      BtPassword := SourceSettings.UsersForm.BtPassword;
      BtClose := SourceSettings.UsersForm.BtClose;
      PromptDelete := SourceSettings.UsersForm.PromptDelete;
    end;
    with Settings.UsersProfile do
    begin
      WindowCaption :=   SourceSettings.UsersProfile.WindowCaption;
      LabelDescription := SourceSettings.UsersProfile.LabelDescription;
      ColProfile := SourceSettings.UsersProfile.ColProfile;
      BtAdd := SourceSettings.UsersProfile.BtAdd;
      BtChange := SourceSettings.UsersProfile.BtChange;
      BtDelete := SourceSettings.UsersProfile.BtDelete;
      BtClose := SourceSettings.UsersProfile.BtClose;
      PromptDelete := SourceSettings.UsersProfile.PromptDelete;
    end;
    with Settings.AddChangeUser do
    begin
      WindowCaption := SourceSettings.AddChangeUser.WindowCaption;
      LabelAdd := SourceSettings.AddChangeUser.LabelAdd;
      LabelChange := SourceSettings.AddChangeUser.LabelChange;
      LabelName := SourceSettings.AddChangeUser.LabelName;
      LabelLogin := SourceSettings.AddChangeUser.LabelLogin;
      LabelEmail := SourceSettings.AddChangeUser.LabelEmail;
      CheckPrivileged := SourceSettings.AddChangeUser.CheckPrivileged;
      BtSave := SourceSettings.AddChangeUser.BtSave;
      BtCancel := SourceSettings.AddChangeUser.BtCancel;
    end;
    with Settings.AddChangeProfile do
    begin
      WindowCaption := SourceSettings.AddChangeProfile.WindowCaption;
      LabelAdd := SourceSettings.AddChangeProfile.LabelAdd;
      LabelChange := SourceSettings.AddChangeProfile.LabelChange;
      LabelName := SourceSettings.AddChangeProfile.LabelName;
      BtSave := SourceSettings.AddChangeProfile.BtSave;
      BtCancel := SourceSettings.AddChangeProfile.BtCancel;
    end;
    with Settings.Rights do
    begin
      WindowCaption := SourceSettings.Rights.WindowCaption;
      LabelUser := SourceSettings.Rights.LabelUser;
      LabelProfile := SourceSettings.Rights.LabelProfile;
      PageMenu := SourceSettings.Rights.PageMenu;
      PageActions := SourceSettings.Rights.PageActions;
      BtUnlock := SourceSettings.Rights.BtUnlock;
      BtLock := SourceSettings.Rights.BtLock;
      BtSave := SourceSettings.Rights.BtSave;
      BtCancel := SourceSettings.Rights.BtCancel;
    end;
    with Settings.ChangePassword do
    begin
      WindowCaption := SourceSettings.ChangePassword.WindowCaption;
      LabelDescription := SourceSettings.ChangePassword.LabelDescription;
      LabelCurrentPassword := SourceSettings.ChangePassword.LabelCurrentPassword;
      LabelNewPassword := SourceSettings.ChangePassword.LabelNewPassword;
      LabelConfirm := SourceSettings.ChangePassword.LabelConfirm;
      BtSave := SourceSettings.ChangePassword.BtSave;
      BtCancel := SourceSettings.ChangePassword.BtCancel;
    end;
    with Settings.CommonMessages.ChangePasswordError do
    begin
      InvalidCurrentPassword := SourceSettings.CommonMessages.ChangePasswordError.InvalidCurrentPassword;
      NewPasswordError := SourceSettings.CommonMessages.ChangePasswordError.NewPasswordError;
      NewEqualCurrent := SourceSettings.CommonMessages.ChangePasswordError.NewEqualCurrent;
      PasswordRequired := SourceSettings.CommonMessages.ChangePasswordError.PasswordRequired;
      MinPasswordLength := SourceSettings.CommonMessages.ChangePasswordError.MinPasswordLength;
      InvalidNewPassword := SourceSettings.CommonMessages.ChangePasswordError.InvalidNewPassword;
    end;
    with Settings.ResetPassword do
    begin
      WindowCaption := SourceSettings.ResetPassword.WindowCaption;
      LabelPassword := SourceSettings.ResetPassword.LabelPassword;
    end;

    with Settings.Log do
    begin
      WindowCaption := SourceSettings.Log.WindowCaption;
      LabelDescription := SourceSettings.Log.LabelDescription;
      LabelUser := SourceSettings.Log.LabelUser;
      LabelDate := SourceSettings.Log.LabelDate;
      LabelLevel := SourceSettings.Log.LabelLevel;
      ColLevel := SourceSettings.Log.ColLevel;
      ColMessage := SourceSettings.Log.ColMessage;
      ColUser := SourceSettings.Log.ColUser;
      ColDate := SourceSettings.Log.ColDate;
      BtFilter := SourceSettings.Log.BtFilter;
      BtDelete := SourceSettings.Log.BtDelete;
      BtClose := SourceSettings.Log.BtClose;
      PromptDelete := SourceSettings.Log.PromptDelete;
    end;

    with Settings.AppMessages do
    begin
      MsgsForm_BtNew := SourceSettings.AppMessages.MsgsForm_BtNew;
      MsgsForm_BtReplay := SourceSettings.AppMessages.MsgsForm_BtReplay;
      MsgsForm_BtForward := SourceSettings.AppMessages.MsgsForm_BtForward;
      MsgsForm_BtDelete := SourceSettings.AppMessages.MsgsForm_BtDelete;
      MsgsForm_WindowCaption := SourceSettings.AppMessages.MsgsForm_WindowCaption;
      MsgsForm_ColFrom := SourceSettings.AppMessages.MsgsForm_ColFrom;
      MsgsForm_ColSubject := SourceSettings.AppMessages.MsgsForm_ColSubject;
      MsgsForm_ColDate := SourceSettings.AppMessages.MsgsForm_ColDate;
      MsgsForm_PromptDelete := SourceSettings.AppMessages.MsgsForm_PromptDelete;

      MsgRec_BtClose := SourceSettings.AppMessages.MsgRec_BtClose;
      MsgRec_WindowCaption := SourceSettings.AppMessages.MsgRec_WindowCaption;
      MsgRec_Title := SourceSettings.AppMessages.MsgRec_Title;
      MsgRec_LabelFrom := SourceSettings.AppMessages.MsgRec_LabelFrom;
      MsgRec_LabelDate := SourceSettings.AppMessages.MsgRec_LabelDate;
      MsgRec_LabelSubject := SourceSettings.AppMessages.MsgRec_LabelSubject;
      MsgRec_LabelMessage := SourceSettings.AppMessages.MsgRec_LabelMessage;

      MsgSend_BtSend := SourceSettings.AppMessages.MsgSend_BtSend;
      MsgSend_BtCancel := SourceSettings.AppMessages.MsgSend_BtCancel;
      MsgSend_WindowCaption := SourceSettings.AppMessages.MsgSend_WindowCaption;
      MsgSend_Title := SourceSettings.AppMessages.MsgSend_Title;
      MsgSend_GroupTo := SourceSettings.AppMessages.MsgSend_GroupTo;
      MsgSend_RadioUser := SourceSettings.AppMessages.MsgSend_RadioUser;
      MsgSend_RadioAll := SourceSettings.AppMessages.MsgSend_RadioAll;
      MsgSend_GroupMessage := SourceSettings.AppMessages.MsgSend_GroupMessage;
    end;

end;


{$IFDEF Ver130}
function StrToBool ( Valor : String) : Boolean;
begin
  if Valor = '-1' then Result := True else Result := False;
end;

function BoolToStr ( Valor : Boolean) : String;
begin
  if Valor then Result := '-1' else Result := '0';
end;
{$ENDIF}

{ TCadastroUsuarios }

procedure TCadastroUsuarios.Assign(Source: TPersistent);
begin
  if Source is TCadastroUsuarios then
  begin
    Self.MenuItem := TCadastroUsuarios(Source).MenuItem;
    Self.Action := TCadastroUsuarios(Source).Action;
  end else inherited;
end;

constructor TCadastroUsuarios.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TCadastroUsuarios.Destroy;
begin
  inherited;
end;


procedure TCadastroUsuarios.SetActCadUsu(const Value: TAction);
begin
  FActCadUsu := Value;
  if Value <> nil then Value.FreeNotification(Self.Action);
end;


procedure TCadastroUsuarios.SetMenuCadUsu(const Value: TMenuItem);
begin
  FMenuCadUsu := Value;
  if Value <> nil then Value.FreeNotification(Self.MenuItem);
end;



{ TLogin }

procedure TLogin.Assign(Source: TPersistent);
begin
  if Source is TLogin then
  begin
    Self.MaxLoginAttempts := TLogin(Source).MaxLoginAttempts;
  end else inherited;
end;

constructor TLogin.Create(AOwner: TComponent);
begin
  inherited Create;
  AutoLogon := TUCAutoLogin.Create(nil);
  InitialLogin := TInitialLogin.Create(nil);
  if not AutoLogon.MessageOnError  then AutoLogon.MessageOnError := True;
end;

destructor TLogin.Destroy;
begin
  inherited;
end;





{ TPerfilUsuarios }

procedure TPerfilUsuarios.Assign(Source: TPersistent);
begin
  if Source is TPerfilUsuarios then
  begin
    Self.MenuItem := TPerfilUsuarios(Source).MenuItem;
    Self.Action := TPerfilUsuarios(Source).Action;
  end else inherited;
end;

constructor TPerfilUsuarios.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TPerfilUsuarios.Destroy;
begin
  inherited;
end;

procedure TPerfilUsuarios.SetActPerfUsu(const Value: TAction);
begin
  FActPerfUsu := Value;
  if Value <> nil then Value.FreeNotification(Self.Action);
end;


procedure TPerfilUsuarios.SetMenuPerfUsu(const Value: TMenuItem);
begin
  FMenuPerfUsu := Value;
  if Value <> nil then Value.FreeNotification(Self.MenuItem);
end;

{ TTrocarSenha }

procedure TTrocarSenha.Assign(Source: TPersistent);
begin
  if Source is TTrocarSenha then
  begin
    Self.MenuItem := TTrocarSenha(Source).MenuItem;
    Self.Action := TTrocarSenha(Source).Action;

    Self.ForcePassword := TTrocarSenha(Source).ForcePassword;
    Self.MinPasswordLength := TTrocarSenha(Source).MinPasswordLength;
  end else inherited;
end;

constructor TTrocarSenha.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TTrocarSenha.Destroy;
begin
  inherited;
end;

procedure TTrocarSenha.SetActChgPassUsu(const Value: TAction);
begin
  FActChgPass := Value;
  if Value <> nil then Value.FreeNotification(Self.Action);
end;


procedure TTrocarSenha.SetMenuChgPassUsu(const Value: TMenuItem);
begin
  FMenuChgPassUsu := Value;
  if Value <> nil then Value.FreeNotification(Self.MenuItem);
end;

{ TInitialLogin }

procedure TInitialLogin.Assign(Source: TPersistent);
begin
  if Source is TInitialLogin then
  begin
    Self.User := TInitialLogin(Source).User;
    Self.Password := TInitialLogin(Source).Password;
  end else inherited;

end;

constructor TInitialLogin.Create(AOwner: TComponent);
begin
  inherited Create;
  FPermINI := TStringList.Create;
end;

destructor TInitialLogin.Destroy;
begin
  inherited;
end;

procedure TInitialLogin.SetFPermIni(const Value: TStrings);
begin
  FPermIni.Assign(Value);
end;

{ TUCActions }

procedure TUCActions.Assign(Source: TPersistent);
begin
  if Source is TUCActions then
  begin
    Self.ActionList := TUCActions(Source).ActionList;
    {$IFDEF VER150}
    Self.ActionManager := TUCActions(Source).ActionManager;
    {$ENDIF}
  end else inherited;

end;

constructor TUCActions.Create(AOwner: TComponent);
begin
  inherited Create;
end;

destructor TUCActions.Destroy;
begin
  inherited;
end;

procedure TUCActions.SetActionList(const Value: TActionList);
begin
  FActionList := Value;
  if Value <> nil then
  begin
    Value.FreeNotification(Self.ActionList);
    {$IFDEF VER150}
    FActionManager := nil;
    {$ENDIF}
  end;
end;
{$IFDEF VER150}
procedure TUCActions.SetActionMainMenuBar(
  const Value: TActionMainMenuBar);
begin
  FActionMainMenuBar := Value;
  if Value <> nil then Value.FreeNotification(Self.ActionMainMenuBar);
end;

procedure TUCActions.SetActionManager(const Value: TActionManager);
begin
  FActionManager := Value;
  if Value <> nil then
  begin
    Value.FreeNotification(Self.ActionManager);
    FActionList := nil;
  end;
end;
{$ENDIF}

{ TUCAppMessage }

procedure TUCAppMessage.CheckMessages;
  function FmtDtHr(dt : String) : String;
  begin
    Result := Copy(dt,7,2)+'/'+Copy(dt,5,2)+'/'+Copy(dt,1,4)+' '+Copy(dt,9,2)+':'+Copy(dt,11,2);
  end;
begin
  if not FReady then Exit;
  with Self.UserControl.UCGetSQLDataset('SELECT UCM.IdMsg, UCC.Nome AS De, UCC_1.Nome AS Para, UCM.Subject, UCM.Msg, UCM.DtSend, UCM.DtReceive ' +
    'FROM ('+ Self.TableMessages+ ' UCM INNER JOIN ' + Self.UserControl.TableUsers + ' UCC ON UCM.UsrFrom = UCC.IdUser) INNER JOIN '+
    Self.UserControl.TableUsers + ' UCC_1 ON UCM.UsrTo = UCC_1.IdUser where UCM.DtReceive is NULL and  UCM.UsrTo = ' + IntToStr(Self.UserControl.CurrentUser.UserID)) do
  begin
    while not eof do
    begin
      MsgRecForm := TMsgRecForm.Create(Application);
      MsgRecForm.stDe.Caption := Fieldbyname('De').asString;
      MsgRecForm.stData.Caption := FmtDtHr(Fieldbyname('DtSend').asString);
      MsgRecForm.stAssunto.Caption := Fieldbyname('Subject').asString;
      MsgRecForm.MemoMsg.Text := Fieldbyname('msg').asString;
      Self.UserControl.UCExecSQL('Update '+ Self.TableMessages + ' set DtReceive =  '+
                                 QuotedStr(FormatDateTime('YYYYMMDDhhmm',now)) +
                                 ' Where  idMsg = ' + fieldbyname('idMsg').asString);
      MsgRecForm.Show;
      next;
    end;
    Close;
    Free;
  end;

end;

constructor TUCAppMessage.Create(AOWner: TComponent);
begin
  inherited;
  FReady := False;
  if csDesigning in ComponentState then
  begin
    Interval := 60000;
    Active := True;
  end;// else begin
    FVerifThread := TVerifThread.Create(True);
    FVerifThread.AOwner := Self;
    FVerifThread.FreeOnTerminate := True;
//  end;

end;

procedure TUCAppMessage.DeleteAppMessage(IdMsg: Integer);
begin
  if MessageDlg(FUserControl.Settings.AppMessages.MsgsForm_PromptDelete, mtConfirmation, [mbYes, mbNo],0) <> mrYes then exit;
  UserControl.UCExecSQL('Delete from '+ TableMessages + ' where IdMsg = '+ IntToStr(idMsg));
end;

destructor TUCAppMessage.Destroy;
begin
  if not (csDesigning in ComponentState) then FVerifThread.Terminate;
  inherited Destroy;
end;

procedure TUCAppMessage.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) then
  begin
    if not Assigned(FUserControl) then raise Exception.Create('Componente User Control nao definido!');
    if not FUserControl.UCFindTable(TableMessages) then FUserControl.CriaTabelaMsgs(TableMessages);
    FVerifThread := TVerifThread.Create(True);
    FVerifThread.AOwner := Self;
    FVerifThread.FreeOnTerminate := True;
  end;
  FReady := True;
end;

procedure TUCAppMessage.SendAppMessage(ToUser: Integer; Subject, Msg: String);
var
  UltId : integer;
begin
  with UserControl.UCGetSQLDataset('Select Max(idMsg) as nr from '+ TableMessages) do
  begin
    UltID := Fieldbyname('nr').asInteger +1;
    Close;
    Free;
  end;
  UserControl.UCExecSQL('Insert into '+ TableMessages + '( idMsg, UsrFrom, UsrTo, Subject, Msg, DtSend) Values ('+
                        IntToStr(UltId)+ ', '+
                        IntToStr(UserControl.CurrentUser.UserID)+', '+
                        IntToStr(toUser)+', '+
                        QuotedStr(Subject)+', '+
                        QuotedStr(Msg)+', '+
                        QuotedStr(FormatDateTime('YYYYMMDDHHMM',now))+')');

end;

procedure TUCAppMessage.SetActive(const Value: Boolean);
begin
  FActive := Value;
  if (csDesigning in ComponentState) then exit;
  if FActive then FVerifThread.Resume else FVerifThread.Suspend;
end;

procedure TUCAppMessage.ShowMessages;
begin
  MsgsForm := TMsgsForm.Create(self);
  with FUserControl.Settings.AppMessages do
  begin
    MsgsForm.Caption := MsgsForm_WindowCaption;
    MsgsForm.btnova.Caption := MsgsForm_BtNew;
    MsgsForm.btResponder.Caption := MsgsForm_BtReplay;
    MsgsForm.btEncaminhar.Caption := MsgsForm_BtForward;
    MsgsForm.btExcluir.Caption := MsgsForm_BtDelete;
    MsgsForm.ListView1.Columns[0].Caption := MsgsForm_ColFrom;
    MsgsForm.ListView1.Columns[1].Caption := MsgsForm_ColSubject;
    MsgsForm.ListView1.Columns[2].Caption := MsgsForm_ColDate;
  end;

  MsgsForm.DSMsgs := UserControl.UCGetSQLDataset('SELECT UCM.IdMsg, UCM.UsrFrom, UCC.Nome AS De, UCC_1.Nome AS Para, UCM.Subject, UCM.Msg, UCM.DtSend, UCM.DtReceive '+
                                   'FROM ('+ TableMessages + ' UCM INNER JOIN '+ UserControl.TableUsers+ ' UCC ON UCM.UsrFrom = UCC.IdUser) '+
                                   ' INNER JOIN '+ UserControl.TableUsers+ ' UCC_1 ON UCM.UsrTo = UCC_1.IdUser WHERE UCM.UsrTo = '+IntToStr(UserControl.CurrentUser.UserID)+' ORDER BY UCM.DtReceive DESC');
  MsgsForm.DSMsgs.Open;
  MsgsForm.DSUsuarios := UserControl.UCGetSQLDataset('SELECT * from '+ UserControl.TableUsers+ ' where IdUser <> ' +IntToStr(UserControl.CurrentUser.UserID)+' ORDER BY Nome');
  MsgsForm.DSUsuarios.Open;

  MsgsForm.ShowModal;
  FreeAndNil(MsgsForm)
end;

{ TVerifThread }

procedure TVerifThread.Execute;
begin
//  Sleep(10000);
//  if TUCAppMessage(AOwner).UserControl.CurrentUser.UserID = 0 then
//  begin
//    Sleep(TUCAppMessage(aowner).Interval);
//    exit;
//  end;
//  if not Assigned(TUCAppMessage(AOwner).UserControl) then
  while not self.Terminated do
  begin
    if (Assigned(TUCAppMessage(AOwner).UserControl)) and
    (TUCAppMessage(AOwner).UserControl.CurrentUser.UserID <> 0) then Synchronize(VerNovaMsg);
    Sleep(TUCAppMessage(aowner).Interval);
  end;
end;

procedure TVerifThread.VerNovaMsg;
{  function FmtDtHr(dt : String) : String;
  begin
    Result := Copy(dt,7,2)+'/'+Copy(dt,5,2)+'/'+Copy(dt,1,4)+' '+Copy(dt,9,2)+':'+Copy(dt,11,2);
  end;}
var
  AOW : TUCAppMessage;
begin
  AOW := TUCAppMessage(AOwner);
  AOW.CheckMessages;
  {
  with AOW.UserControl.UCGetSQLDataset('SELECT UCM.IdMsg, UCC.Nome AS De, UCC_1.Nome AS Para, UCM.Subject, UCM.Msg, UCM.DtSend, UCM.DtReceive ' +

    'FROM ('+ AOW.TableMessages+ ' UCM INNER JOIN ' + AOW.UserControl.TableUsers + ' UCC ON UCM.UsrFrom = UCC.IdUser) INNER JOIN '+
    AOW.UserControl.TableUsers + ' UCC_1 ON UCM.UsrTo = UCC_1.IdUser where UCM.DtReceive is NULL and  UCM.UsrTo = ' + IntToStr(AOW.UserControl.CurrentUser.UserID)) do
  begin
    while not eof do
    begin
      MsgRecForm := TMsgRecForm.Create(Application);
      MsgRecForm.stDe.Caption := Fieldbyname('De').asString;
      MsgRecForm.stData.Caption := FmtDtHr(Fieldbyname('DtSend').asString);
      MsgRecForm.stAssunto.Caption := Fieldbyname('Subject').asString;
      MsgRecForm.MemoMsg.Text := Fieldbyname('msg').asString;
      AOW.UserControl.UCExecSQL('Update '+ AOW.TableMessages + ' set DtReceive =  '+
                                 QuotedStr(FormatDateTime('YYYYMMDDhhmm',now)) +
                                 ' Where  idMsg = ' + fieldbyname('idMsg').asString);
      MsgRecForm.Show;
      next;
    end;
    Close;
    Free;
  end;}
end;

procedure Register;
begin
  RegisterComponents('User Control', [TUCAppMessage]);
  RegisterComponents('User Control', [TUCSettings]);
  RegisterPropertyEditor( TypeInfo( TUCAboutVar ), nil, '', TUCAboutVarProperty );  
end;

{ TUCSettings }

procedure TUCSettings.Assign(Source: TPersistent);
begin
  if Source is TUserSettings then begin
    Self.CommonMessages := TUserSettings(Source).CommonMessages;
    Self.AppMessages := TUserSettings(Source).AppMessages;
  end else inherited;
end;

constructor TUCSettings.Create(AOwner: TComponent);
begin
  inherited;
  FAppMessagesMSG := TAppMessagesMSG.Create(nil);
  FLoginFormMSG := TLoginFormMSG.Create(nil);
  FUserCommomMSG := TUserCommonMSG.Create(nil);
  FCadUserFormMSG := TCadUserFormMSG.Create(nil);
  FAddUserFormMSG := TAddUserFormMSG.Create(nil);
  FAddProfileFormMSG := TAddProfileFormMSG.Create(nil);
  FPermissFormMSG := TPermissFormMSG.Create(nil);
  FProfileUserFormMSG := TProfileUserFormMSG.Create(nil);
  FTrocaSenhaFormMSG := TTrocaSenhaFormMSG.Create(nil);
  FResetPassword := TResetPassword.Create(nil);
  FLogControlFormMSG := TLogControlFormMSG.Create(nil);

  if csDesigning in ComponentState then IniSettings2(Self);

end;

destructor TUCSettings.Destroy;
begin
  inherited;
end;



procedure TUCSettings.SetAppMessagesMSG(const Value: TAppMessagesMSG);
begin
  FAppMessagesMSG := Value;
end;

procedure TUCSettings.SetFAddProfileFormMSG(
  const Value: TAddProfileFormMSG);
begin
  FAddProfileFormMSG := Value;
end;

procedure TUCSettings.SetFAddUserFormMSG(const Value: TAddUserFormMSG);
begin
  FAddUserFormMSG := Value;
end;

procedure TUCSettings.SetFCadUserFormMSG(const Value: TCadUserFormMSG);
begin
  FCadUserFormMSG := Value;
end;

procedure TUCSettings.SetFFormLoginMsg(const Value: TLoginFormMSG);
begin
  FLoginFormMSG := Value;
end;

procedure TUCSettings.SetFLogControlFormMSG(
  const Value: TLogControlFormMSG);
begin
  FLogControlFormMSG := Value;
end;

procedure TUCSettings.SetFPermissFormMSG(const Value: TPermissFormMSG);
begin
  FPermissFormMSG := Value;
end;

procedure TUCSettings.SetFProfileUserFormMSG(
  const Value: TProfileUserFormMSG);
begin
  FProfileUserFormMSG := Value;
end;

procedure TUCSettings.SetFResetPassword(const Value: TResetPassword);
begin
  FResetPassword := Value;
end;

procedure TUCSettings.SetFTrocaSenhaFormMSG(
  const Value: TTrocaSenhaFormMSG);
begin
  FTrocaSenhaFormMSG := Value;
end;

procedure TUCSettings.SetFUserCommonMSg(const Value: TUserCommonMSG);
begin
  FUserCommomMSG := Value;
end;


procedure IniSettings2(DestSettings: TUCSettings);
var
  tmp : TBitmap;
begin
    with DestSettings.CommonMessages do
    begin
      if BlankPassword = '' then BlankPassword := Const_Men_SenhaDesabitada;
      if PasswordChanged = '' then PasswordChanged := Const_Men_SenhaAlterada;
      if InitialMessage.Text = '' then InitialMessage.Text := Const_Men_MsgInicial;
      if MaxLoginAttemptsError = '' then MaxLoginAttemptsError := Const_Men_MaxTentativas;
      if InvalidLogin = '' then InvalidLogin := Const_Men_LoginInvalido;
      if AutoLogonError = '' then AutoLogonError := Const_Men_AutoLogonError;
    end;
    with DestSettings.Login do
    begin
      if BtCancel = '' then BtCancel := Const_Log_BtCancelar;
      if BtOK = '' then BtOK := Const_Log_BtOK;
      if LabelPassword = '' then LabelPassword := Const_Log_LabelSenha;
      if LabelUser = '' then LabelUser := Const_Log_LabelUsuario;
      if WindowCaption = '' then WindowCaption := Const_Log_WindowCaption;

      Tmp := TBitmap.create;
      Tmp.LoadFromResourceName(HInstance, 'UCLOCKLOGIN');
      LeftImage.Assign(tmp);

    end;
    with DestSettings.UsersForm do
    begin
      if WindowCaption = '' then WindowCaption := Const_Cad_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Cad_LabelDescricao;
      if ColName = '' then ColName := Const_Cad_ColunaNome;
      if ColLogin = '' then ColLogin := Const_Cad_ColunaLogin;
      if ColEmail = '' then ColEmail := Const_Cad_ColunaEmail;
      if BtAdd = '' then BtAdd := Const_Cad_BtAdicionar;
      if BtChange = '' then BtChange := Const_Cad_BtAlterar;
      if BtDelete = '' then BtDelete := Const_Cad_BtExcluir;
      if BtRights = '' then BtRights := Const_Cad_BtPermissoes;
      if BtPassword = '' then BtPassword := Const_Cad_BtSenha;
      if BtClose = '' then BtClose := Const_Cad_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_Cad_ConfirmaExcluir;
    end;
    with DestSettings.UsersProfile do
    begin
      if WindowCaption = '' then WindowCaption := Const_Prof_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Prof_LabelDescricao;
      if ColProfile = '' then ColProfile := Const_Prof_ColunaNome;
      if BtAdd = '' then BtAdd := Const_Prof_BtAdicionar;
      if BtChange = '' then BtChange := Const_Prof_BtAlterar;
      if BtDelete = '' then BtDelete := Const_Prof_BtExcluir;
      if BtRights = '' then BtRights := Const_Prof_BtPermissoes;
      if BtClose = '' then BtClose := Const_Prof_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_Prof_ConfirmaExcluir;
    end;
    with DestSettings.AddChangeUser do
    begin
      if WindowCaption = '' then WindowCaption := Const_Inc_WindowCaption;
      if LabelAdd = '' then LabelAdd := Const_Inc_LabelAdicionar;
      if LabelChange = '' then LabelChange := Const_Inc_LabelAlterar;
      if LabelName = '' then LabelName := Const_Inc_LabelNome;
      if LabelLogin = '' then LabelLogin := Const_Inc_LabelLogin;
      if LabelEmail = '' then LabelEmail := Const_Inc_LabelEmail;
      if CheckPrivileged = '' then CheckPrivileged := Const_Inc_CheckPrivilegiado;
      if BtSave = '' then BtSave := Const_Inc_BtGravar;
      if BtCancel = '' then BtCancel := Const_Inc_BtCancelar;
    end;
    with DestSettings.AddChangeProfile do
    begin
      if WindowCaption = '' then WindowCaption := Const_PInc_WindowCaption;
      if LabelAdd = '' then LabelAdd := Const_PInc_LabelAdicionar;
      if LabelChange = '' then LabelChange := Const_PInc_LabelAlterar;
      if LabelName = '' then LabelName := Const_PInc_LabelNome;
      if BtSave = '' then BtSave := Const_PInc_BtGravar;
      if BtCancel = '' then BtCancel := Const_PInc_BtCancelar;
    end;
    with DestSettings.Rights do
    begin
      if WindowCaption = '' then WindowCaption := Const_Perm_WindowCaption;
      if LabelUser = '' then LabelUser := Const_Perm_LabelUsuario;
      if LabelProfile = '' then LabelProfile := Const_Perm_LabelPerfil;
      if PageMenu = '' then PageMenu := Const_Perm_PageMenu;
      if PageActions = '' then PageActions := Const_Perm_PageActions;
      if BtUnlock = '' then BtUnlock := Const_Perm_BtLibera;
      if BtLock = '' then BtLock := Const_Perm_BtBloqueia;
      if BtSave = '' then BtSave := Const_Perm_BtGravar;
      if BtCancel = '' then BtCancel := Const_Perm_BtCancelar;
    end;
    with DestSettings.ChangePassword do
    begin
      if WindowCaption = '' then WindowCaption := Const_Troc_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_Troc_LabelDescricao;
      if LabelCurrentPassword = '' then LabelCurrentPassword := Const_Troc_LabelSenhaAtual;
      if LabelNewPassword = '' then LabelNewPassword := Const_Troc_LabelNovaSenha;
      if LabelConfirm = '' then LabelConfirm := Const_Troc_LabelConfirma;
      if BtSave = '' then BtSave := Const_Troc_BtGravar;
      if BtCancel = '' then BtCancel := Const_Troc_BtCancelar;
    end;
    with DestSettings.CommonMessages.ChangePasswordError do
    begin
      if InvalidCurrentPassword = '' then InvalidCurrentPassword :=  Const_ErrPass_SenhaAtualInvalida;
      if NewPasswordError = '' then NewPasswordError :=  Const_ErrPass_ErroNovaSenha;
      if NewEqualCurrent = '' then NewEqualCurrent :=  Const_ErrPass_NovaIgualAtual;
      if PasswordRequired = '' then PasswordRequired :=  Const_ErrPass_SenhaObrigatoria;
      if MinPasswordLength = '' then MinPasswordLength := Const_ErrPass_SenhaMinima;
      if InvalidNewPassword= '' then InvalidNewPassword :=  Const_ErrPass_SenhaInvalida;
    end;
    with DestSettings.ResetPassword do
    begin
      if WindowCaption = '' then WindowCaption := Const_DefPass_WindowCaption;
      if LabelPassword = '' then LabelPassword := Const_DefPass_LabelSenha;
    end;
    with DestSettings.Log do
    begin
      if WindowCaption = '' then WindowCaption := Const_LogC_WindowCaption;
      if LabelDescription = '' then LabelDescription := Const_LogC_LabelDescricao;
      if LabelUser = '' then LabelUser := Const_LogC_LabelUsuario;
      if LabelDate = '' then LabelDate := Const_LogC_LabelData;
      if LabelLevel = '' then LabelLevel := Const_LogC_LabelNivel;
      if ColLevel = '' then ColLevel := Const_LogC_ColunaNivel;
      if ColMessage = '' then ColMessage := Const_LogC_ColunaMensagem;
      if ColUser = '' then ColUser := Const_LogC_ColunaUsuario;
      if ColDate = '' then ColDate := Const_LogC_ColunaData;
      if BtFilter = '' then BtFilter := Const_LogC_BtFiltro;
      if BtDelete = '' then BtDelete := Const_LogC_BtExcluir;
      if BtClose = '' then BtClose := Const_LogC_BtFechar;
      if PromptDelete = '' then PromptDelete := Const_LogC_ConfirmaExcluir;
    end;
    with DestSettings.AppMessages do
    begin
      if MsgsForm_BtNew = '' then MsgsForm_BtNew := Const_Msgs_BtNew;
      if MsgsForm_BtReplay = '' then MsgsForm_BtReplay := Const_Msgs_BtReplay;
      if MsgsForm_BtForward = '' then MsgsForm_BtForward := Const_Msgs_BtForward;
      if MsgsForm_BtDelete = '' then MsgsForm_BtDelete := Const_Msgs_BtDelete;
      if MsgsForm_WindowCaption = '' then MsgsForm_WindowCaption := Const_Msgs_WindowCaption;
      if MsgsForm_ColFrom = '' then  MsgsForm_ColFrom := Const_Msgs_ColFrom;
      if MsgsForm_ColSubject = '' then  MsgsForm_ColSubject := Const_Msgs_ColSubject;
      if MsgsForm_ColDate = '' then MsgsForm_ColDate := Const_Msgs_ColDate;
      if MsgsForm_PromptDelete = '' then  MsgsForm_PromptDelete := Const_Msgs_PromptDelete;

      if MsgRec_BtClose = '' then  MsgRec_BtClose := Const_MsgRec_BtClose;
      if MsgRec_WindowCaption = '' then MsgRec_WindowCaption := Const_MsgRec_WindowCaption;
      if MsgRec_Title = ''then MsgRec_Title := Const_MsgRec_Title;
      if MsgRec_LabelFrom = ''then MsgRec_LabelFrom := Const_MsgRec_LabelFrom;
      if MsgRec_LabelDate = '' then MsgRec_LabelDate := Const_MsgRec_LabelDate;
      if MsgRec_LabelSubject = '' then MsgRec_LabelSubject := Const_MsgRec_LabelSubject;
      if MsgRec_LabelMessage = '' then MsgRec_LabelMessage := Const_MsgRec_LabelMessage;

      if MsgSend_BtSend =  '' then MsgSend_BtSend := Const_MsgSend_BtSend;
      if MsgSend_BtCancel = '' then MsgSend_BtCancel := Const_MsgSend_BtCancel;
      if MsgSend_WindowCaption = '' then MsgSend_WindowCaption := Const_MsgSend_WindowCaption;
      if MsgSend_Title = '' then MsgSend_Title := Const_MsgSend_Title;
      if MsgSend_GroupTo = '' then MsgSend_GroupTo := Const_MsgSend_GroupTo;
      if MsgSend_RadioUser = '' then MsgSend_RadioUser := Const_MsgSend_RadioUser;
      if MsgSend_RadioAll = '' then MsgSend_RadioAll := Const_MsgSend_RadioAll;
      if MsgSend_GroupMessage = '' then MsgSend_GroupMessage := Const_MsgSend_GroupMessage;
    end;
end;


{ TUCAboutVarProperty }

procedure TUCAboutVarProperty.Edit;
begin
  with TAboutForm.Create(nil) do
  begin
    ShowModal;
    Free;
  end;
end;

function TUCAboutVarProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TUCAboutVarProperty.GetValue: String;
begin
  result:='About...';
end;

end.
